#!/usr/bin/perl -w

# GG 06/07/2002
# Usage: gg-job-submit.pl toslac.jdl [index]
# Patched for special EDG-1.2.0 (23/08/02)
# Improved for Aborted within EDG-1.4.0 (14/01/03)
# Adapted to TSTG TB with EDG-1.4.3 (GG, 30/01/03)
# Work in progress (GG, 061003)

use POSIX qw(:signal_h :errno_h :sys_wait_h);
use File::Basename;
use IO::File;
use strict;

=head1 NAME

SubmitUtils.pm - Specific to the Stack subclass.
# $Id: SubmitUtils.pm,v 1.9 2003/10/06 13:13:11 grodid Exp $

=head1 SYNOPSIS

This utility script is specific to the Stack subclass,
and MUST NOT be called directly.

=head1 DESCRIPTION 

B<SubmitUtils.pm> This specific utility script is used by the Stack subclass,
for generating the Storms. It must not be called directly in any way.
It is not a user callable function.

 Please refer to BaseTest::EDGLifecycle::Stack and/or to
BaseTest::EDGLifecycle::Stack::JobStorm for simple examples of Storms.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

STDOUT->autoflush(1);
STDERR->autoflush(1);

my $jdl  = $ARGV[0];

# my $date = "@" . `date`;
# chomp $date;
# $date =~ s/\s+/_/g;
# $date .= "@";
chomp(my $date = `date`);
$date =~ s/\s+/_/g;

## Several sleeping times and polling periods ...
my $sleepT = $ENV{STACK_POLLING_PERIOD};
my $timeoutReq = $ENV{STACK_CHILD_TIMEOUT} - 2*$sleepT;
my $timeoutStat = 30;

#p print "\n Jdl: $jdl \n\n";
# my $cmds = "dg-job-submit";
# my $cmdt = "dg-job-status";
# my $cmdo = "dg-job-get-output";
# my $cmdl = "dg-job-get-logging-info";
my $cmds = dgjob("submit");
my $cmdt = dgjob("status");
my $cmdo = dgjob("get-output");
my $cmdl = dgjob("get-logging-info");
my $stdOutput = `grep StdOutput $jdl | cut -d'"' -f2`;
chomp $stdOutput;
my $stdError = `grep StdError $jdl | cut -d'"' -f2`;
chomp $stdError;
#p print " StdOutput: $stdOutput \n";
my $bigFileName = `grep BigFileName $jdl | cut -d'"' -f2` || "";
chomp $bigFileName;
my $config = "";
my $index = 0;
my @mstore = ();
my $mflag = 0;
my $jobId;
my $debugF = 0;
my $logFile;
my $outFile;

my $Aserie = "S";
my $Aindex = "I";

if ( $#ARGV > 0 ) {
    $index = $ARGV[1];
    $Aindex .= $index;
    $logFile = $ARGV[2];
    $outFile = $ARGV[3];
}
if ( exists $ENV{STACK_SERIE} ) {
  $Aserie .= $ENV{STACK_SERIE};
}
## Local temporary dir
my $localDir = dirname($jdl);

## Specific to LCG0 ...
#NONEED$ENV{EDG_WL_UI_CONFIG_PATH} =  "/afs/cern.ch/user/g/grodid/public/grid/MST-220403/UI_ConfigENV-0246.cfg";

my $sigusr1 = 0;
# $SIG{USR1} = sub { print STDERR " Signal USR1 received0: $$ \n"; $sigusr1++; };
# $SIG{ALRM} = sub { print STDERR " Signal ALRM received0: $$ \n"; $sigusr1++; };
$SIG{USR1} = sub { print STDERR " Signal USR1 received0: $$ \n"; die "Timeout in SubmitUtils:"; };
## Only this one seems to be useful !
$SIG{ALRM} = sub { print STDERR " Signal ALRM received0: $$ \n"; 
#best		   printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
		   printRES (" flag=>Status, stat2=>TIMEOUT, stat1=>[FAIL]\n");
		   die "Timeout in SubmitUtils:"; };
##good$SIG{ALRM} = sub { print STDERR " Signal ALRM received0: $$ \n"; die "Timeout in SubmitUtils:"; };

## Slac specific, left as an example
if (0) {
if ( $#ARGV > 1 ) { 
  $config = $ARGV[2];
  print " Config: _${config}_ \n";
  if ( $config eq "gw39" ) {
    $config = "-config /u/eb/grodid/public/globus/grodid/edg-test/UI_ConfigENV_gw39.cfg";
  } elsif ( $config eq "cern" ) {
    $config = "-config /u/eb/grodid/public/globus/grodid/edg-test/UI_ConfigENV_cern.cfg";
  } elsif ( $config eq "gm04" ) {
    $config = "-config /u/eb/grodid/public/globus/grodid/edg-test/UI_ConfigENV_gm04.cfg";
  }
}
}

## Inserting the variable Arguments into the jdl file
## NO MORE NEEDED: done in ggstack ...
##oldmy $njdl = "/tmp/$ENV{USER}-$$-" . basename(${jdl});
my $njdl = "/tmp/$ENV{USER}-$$-" . "temp.jdl";
if (0) { ##old
  my $bid = `grep -v Arguments $jdl > $njdl`;
  $bid = `echo 'Arguments = "${Aserie} ${Aindex} rien"; ' >> $njdl`;
  $jdl = $njdl;
} else { ##new
#  my $bid = `cat $jdl > $njdl`;
## we are NOW using directly $jdl ...
}

if ( $jdl =~ /toslac3.jdl/ ) {
##good    open SUB, "$cmds --resource bbr-gate03.slac.stanford.edu/jobmanager-lsf-medium $jdl $config 2> /dev/null |";
    open SUB, "$cmds $jdl $config 2> /dev/null |";
#p    print "$cmds --resource bbr-gate03.slac.stanford.edu/jobmanager-lsf-medium $jdl $config 2> /dev/null |\n";
} else {
##good  open SUB, "$cmds --resource bbr-gate01.slac.stanford.edu/jobmanager-lsf-medium $jdl $config 2> /dev/null |";
    open SUB, "$cmds $jdl $config 2> /dev/null |";
}

## For debugging the environment
if (0) {
#    print STDERR `env | egrep 'opt|edg' `;
    print STDERR " ENV_SUB: ", `env `;
#    print STDERR ` bash -c set `;
}

while (<SUB>) {
    push @mstore, $_;
## For debugging the submission
if (0) {
    print STDERR " SUB Seen: $_";
}

## Special to simulate breaks or mistakes ... ================
    if ( $index == 11 && $debugF ) {
      $_ =~ s/Connecting/Error:/;
    }
## Special to simulate breaks or mistakes ... ================

  if ( /Error:/ ) {
#      open RES, ">> resume-loop.out";
##CTB      print RES " $index $jdl $date $_";
    chomp ( my $line = $_ );
##      printRES (" Trying submission $index -$config- $jdl $date $line UNSUBMITTED [UNTESTED]\n");
      printRES (" flag=>Trying_submission, index=>$index, none=>-$config-, jdl=>$jdl, date=>$date, mess=>$line, stat2=>UNSUBMITTED, stat1=>[UNTESTED]\n");
#      close RES;
      $mflag = 1;
##edg2.0    last;
  }
  if ( /https/ ) {
    my @m = split /\s+/;
    $jobId = $m[2];
    print "\n JobId: $index $jdl $date $jobId \n\n";
#    open RES, ">> resume-loop.out";
##    printRES (" Trying submission sJobId: $index $date $jobId \n");
    printRES (" flag=>Trying_submission, index=>$index, date=>$date, jobid=>$jobId\n");
#    close RES;
    last;
  }
}
close SUB;
## Removing the temporary jdl file
##temp $bid = `rm $njdl`;
#maybenotafterall unlink $jdl;

if ( ! defined $jobId ) { 
    print " SUB Warning: Error_${index}_$date \n";
## Disabled on CTB ## Not any more
if (1) {
#old    open MES, "| mail -s 'Error_${index}_$date' $ENV{USER}";
    open MES, "> $logFile";
    print MES @mstore;
    close MES;
}
    exit 1; 
}

my $dt     = $sleepT;
my $wait   = 0;
my $rct    = 0;
my $rcc;
my $done   = "NOT";
my $nready = 0;
my %nr     = ();
my $ndone  = 0;
my $stat2 = "";
my @staStore = ();
my $cmdFile = "";
my $STAFlag = 0;
WSTA: while (1) {
    @mstore = ();
    $mflag = 0;
    my $sleep = `sleep $dt`;
    $wait += $dt;

    while ( $mflag <= 5 ) {
	($rct, $rcc, $cmdFile, @mstore) = timeoutWithFork("$cmdt $jobId $config 2> /dev/null", $timeoutStat, $cmdFile);
	last if ( $rct == 0 );
	$mflag++;
    }
    if ( $rct != 0 ) {
	$stat2 = "TIMEOUTSTAT";
	printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
	##exit 1;
	subExit(1, "", $cmdFile);
    }
    ## GG 07/06/03
    if ( $rcc != 0 ) {
#p	print STDOUT " STATUS: $rcc \n@mstore\n";
    open MES, "> $logFile";
    print MES @mstore;
    close MES;
	$stat2 = "DEADSTAT";
	printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
	subExit(1, "", $cmdFile);
    }

    ## GG 12/09/03
#deb    my $mail = `echo "@mstore" | mail -s "Status_$index" grodid`;
    my $flagLoc = 0;
    foreach ( @mstore ) {
	my @m = split /\s+/;
	next unless $m[2];
	if ( $m[1] eq "Status:" && $m[2] eq "Done" && $m[3] eq "(Success)" ) { $flagLoc++; }
	next unless $m[3];
	if ( $m[1] eq "Reason:" && $m[2] eq "(nil)" ) { $flagLoc++; }
    }
    if ( $flagLoc == 2 ) { next WSTA; }

##STA  open STA, "$cmdt $jobId $config 2> /dev/null |";
##STA  while (<STA>) {
##STA      push @mstore, $_;
#p    print " STA Seen: $_";
    foreach ( @mstore ) {
    if ( /Status/ ) {
      my @m = split /\s+/;
##edg2.0      if ( $m[1] ne "=" ) { next; }
      if ( $m[1] ne "Status:" ) { next; }
      push @staStore, " STA  Status: $index $jdl $date $wait $m[2] \n";
#       if ( $m[2] =~ /Scheduled|Submitted/ ) { 
# 	  if ( $nready > 15 ) {
# 	      if ( $dt*$nready++ >= $timeoutReq ) { $m[2] = "Aborted"; }
# 	      else { next; }
# 	  } else { next; }
#       } elsif ( $m[2] eq "Done" ) {
#           if ( $nready > 15 ) {
#               if ( $dt*$nready++ >= $timeoutReq ) { $m[2] = "Aborted"; }
# 	  }
#       }

#good      print " STA  Status: $index $jdl $date $wait $m[2] \n";
      print " STA  Status: $index $wait @m[2..$#m] \n"; ## if ( $STAFlag++ == 0 );
#long      print " STA  Status: $index $jdl $date $wait @m[2..$#m] \n";
#trop      print " STA  Status: $index $jdl $date $wait $m[2] \n";

#       if ( $m[2] eq "Ready" ) { 
# 	if ( $dt*$nready++ < $timeoutReq ) { next; }
# 	$m[2] = "Aborted";
#       } elsif ( $m[2] eq "Done" ) {
#         if ( $dt*$ndone++ < $timeoutReq ) { next; }
#         $m[2] = "Aborted";
#       }

      ## GG 12/09/03
      if ( $m[2] eq "Done" && $m[3] eq "(Success)" ) { $m[2] = "Done"; }
      elsif ( $m[2] eq "Done" && $m[3] eq "(Cancelled)" ) { 
	  $m[2] = "Aborted";
	  $stat2 = "CANCELLED";
      }

#edg2.0      if ( $m[2] =~ /Scheduled|Submitted|Ready|Done|Running|Waiting/ && $m[2] ne "OutputReady" ) { 
      if ( $m[2] =~ /Scheduled|Submitted|Ready|Running|Waiting/ && $m[2] ne "Done" ) { 
        if ( $dt*$nr{$m[2]}++ < $timeoutReq ) { next WSTA; }
	$stat2 = "TIMEOUTREQ";
        $m[2] = "Aborted";
      }

## Special to simulate breaks or mistakes ... ================
      if ( $index == 17 && $debugF ) {
      	$m[2] = "Aborted";
      }
## Special to simulate breaks or mistakes ... ================

## To allow for stopping thru USR1 ... ================
      if ( $sigusr1 != 0 ) {
	  print STDERR " Signal USR1 received1: $$ \n";
      	$m[2] = "Aborted";
      }
## To allow for stopping thru USR1 ... ================

##edg2.0      if ( $m[2] eq "Aborted" || $m[2] eq "OutputReady" ) {
      if ( $m[2] eq "Aborted" || $m[2] eq "Done" ) {
	$done = $m[2];
	if ( $done eq "Aborted" ) {
##STA	    my $temp = <STA>;
##STA	    push @mstore, $temp;
##STA	    chomp $temp;
##	    printRES (" $index $jdl $date $wait $temp ABORTED [FAIL]\n");
	    $stat2 = "ABORTED" if ( $stat2 eq "" );
	    if ( $sigusr1 != 0 ) { $stat2 = "TIMEOUT"; };
	    printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, stat2=>$stat2, stat1=>[FAIL]\n");
#	    close RES;
####	    $mflag = 1;
	}
	last;
      }
      last;
    }
  }
##STA  close STA;
  if ( $done ne "NOT" ) { last; }
} 

if ( $done eq "Aborted" ) { 
#prev    open MES, "| mail -s 'Aborted_${index}_$date' $ENV{USER}";
    open MES, "> $logFile";
    print MES "+0+--------------------------------------\n";
    print MES @staStore;
    print MES "+1+--------------------------------------\n";
    print MES @mstore;
    @mstore = `$cmdl $jobId $config 2> /dev/null `;
    print MES "+2+--------------------------------------\n";
    print MES @mstore;
    print MES "+3+--------------------------------------\n";
    close MES;
#    exit 1; 
    subExit(1, "", $cmdFile);
}

$mflag = 0;
my $outName = "nothing";
my $errName = "nothing";
my $dirName = "nothing";
my $bigName = "nothing";

GETOUT: while ( $mflag <= 5 ) {
    @mstore = ();
    open GET, "$cmdo -dir $localDir $jobId $config 2> /dev/null |";
    while (<GET>) {
	push @mstore, $_;
##edg2.0	if ( /RB_API_ERR/ ) {
	if ( /NS_JOB_OUTPUT_NOT_READY/ ) {
	    my $sl = `sleep $sleepT`;
	    $mflag++;
	    next GETOUT;
	}
	if ( /directory:/ ) {
	    my $temp = <GET>;
	    push @mstore, $temp;
	    my @m = split /\s+/, $temp;
	    $outName = $m[1];
	    $dirName = $outName;
	    $errName = "$outName" . "/$stdError";
	    $outName .= "/$stdOutput";
	    $bigName = $dirName . "/$bigFileName";
##edg2.0    last;
	}
    }
    close GET;
    last;
}

# my $errLines = 
# my $outLines = 
my $statLines;

if ( $outName ne "nothing" ) {
my $errLines = `wc $errName | awk '{print \$1}'`;
my $outLines = `wc $outName | awk '{print \$1}'`;
$statLines = " "; 
if ( $outLines*$errLines == 0 ) { 
    $statLines = "empty";
    if ( $errLines == 0 ) { $statLines = "err_" . "$statLines"; }
    if ( $outLines == 0 ) { $statLines = "out_" . "$statLines"; }
}
}

## Special to simulate a break ================
if ( $index == 23 && $debugF ) {
  $outName = "nothing";
}
## Special to simulate a break ================
    
print "\n GET OutName: $index $jdl $date $wait $outName \n";
if ( $outName eq "nothing" || $statLines ne " " ) { 
##    printRES (" GET Warning: nothing_${index}_${date}_${statLines}_${mflag} MISSING_FILES [FAIL]\n");
    printRES (" flag=>Warning, index=>$index, mess=>nothing_${index}_${date}_${statLines}_${mflag}, stat2=>MISSING_FILES, stat1=>[FAIL]\n");
##edg2.0    open MES, "| mail -s 'nothing_${index}_${date}_${statLines}_${mflag}' $ENV{USER}";
    open MES, "> $logFile";
    print MES @mstore;
    close MES;
##edg2.0    if ( $outName eq "nothing" ) { 
    if ( $outName eq "nothing" || $statLines eq "out_empty" ) { 
# 	my $rmes = ` rm -r $dirName `;
# 	exit 2; 
	subExit(2, $dirName, $cmdFile);
    }
}

my $resC = `cat $outName > $outFile`;
my $checksum1;
my $checksum2;
my $checksum3;
my $checksum;
my $stat1;
# my $stat2;
open OUT, "$outName";
my $nlo = 0;
my $host;

if ( -s $bigName && ! -d $bigName ) {

    # Reverse
    my $bigNamR = "$dirName/mumbojomba";
    open BGO, "$bigName";
    open MBJ, "> $bigNamR";
    my @lines = reverse <BGO>;
    foreach my $line (@lines) {
	print MBJ "$line";
    }
    close BGO;
    close MBJ;

    chomp($checksum3 = `cksum $bigNamR | cut -d" " -f1-2`);
    $checksum3 =~ s/ /-/;
    $checksum3 = "=$checksum3=";
    $resC = `echo '\n' CrossCheck local to submitting node: >> $outFile`;
    $resC = `echo =checksum3= $checksum3 >> $outFile`;
    $resC = `echo $dirName $localDir $bigName $bigFileName >> $outFile`;
    $resC = `ls -l $dirName >> $outFile`;
}

while (<OUT>) {
#NOTRANSFERTOOBIG  print " OUT $_";
  if ( /address/ ) {
      next if $host;
      my @m = split /\s+/;
      $host = $m[0];
##      printRES (" Outname: $index $outName $host\n");
      printRES (" flag=>Outname, index=>$index, olout=>$outFile, mawn=>$host\n");
      next;
  }

  if ( /Linux/ ) {

## Special to simulate a break ================
    if ( $index == 25 && $debugF ) {
      $nlo = 0; last;
    }
## Special to simulate a break ================

## Checksum crosscheck ...
    if ( $checksum1 && $checksum2 && $checksum3 ) {
	if ( ($checksum1 eq $checksum2) && ($checksum2 eq $checksum3) ) { 
	    $stat1 = '[OK]'; 
	    $stat2 = 'SUCCESS';
	} else { 
	    $stat1 = '[FAIL]'; 
	    $stat2 = 'CHECKSUM';
	}
    } else {
	$stat1 = '[OK]';
	$stat2 = 'SUCCESS';
    }

      my @m = split /\s+/;
#      open RES, ">> resume-loop.out";
##      printRES (" $index $jdl $date $wait $m[1] $m[2] $statLines SUCCESS [OK]\n");
##edg2.0      printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, none=>$m[1], none=>$m[2], none=>$statLines, stat2=>SUCCESS, stat1=>[OK]\n");
      printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, none=>$m[1], none=>$m[2], none=>$statLines, stat2=>$stat2, stat1=>$stat1\n");
##temp      printRES (" Outname: $index $outName \n");
#      close RES;
    next;
  }

  if ( /=checksum1=/ ) { my @m = split /\s+/; $checksum1 = $m[1]; }
  if ( /=checksum2=/ ) { my @m = split /\s+/; $checksum2 = $m[1]; }

  $nlo++;

}
close OUT;
##maybenot my $rmes = ` rm -r $dirName `;

if ( $nlo <= 1 ) {
#    open RES, ">> resume-loop.out";
##    printRES (" $index $jdl $date $wait $statLines PARTFAIL [FAIL]\n");
    printRES (" flag=>Status, index=>$index, jdl=>$jdl, date=>$date, none=>$wait, none=>$statLines, stat2=>PARTFAIL, stat1=>[FAIL]\n");
#    close RES;
}

print "\n";

#exit 0;
subExit(0, $dirName, $cmdFile);

sub subExit {
    my ($rc, $dirName, $cmdFile) = @_;
##edg2.0    my $rmes = ` rm -r $dirName ` unless ( $dirName eq "" );
    unlink $cmdFile unless ( $cmdFile eq "" );
    exit $rc;
}


sub printRES
  {
    my $str = shift;
    if (0) {
      open RES, ">> resume-loop.out";
      print RES $str;
      close RES;
    } else {
      print " Summary $str";
    }
    return 0;
  }


# This will execute the given command subject to the given timeout (in
# seconds).  It returns an array reference with the first element
# being the return code and the second a scalar variable with the
# command output.
sub timeoutWithRC {
    my ($command, $timeout) = @_;
    $timeout ||= 60;

    my $rc = 0;
    my $crc = 0;
    my @output = ();

    my $r = rand;

    eval {
        local $SIG{ALRM} = sub {$rc = 1; die " Timeout in gg-job" }; 
        alarm $timeout;
        @output = `$command`;
        $crc = $?;
        alarm 0;
    };

#gg    return [$rc, $crc, $output];
    return ($rc, $crc, @output);
}


# This will execute the given command subject to the given timeout (in
# seconds).  It returns an array reference with the first element
# being the return code and the second a scalar variable with the
# command output.
sub timeoutWithFork {
    my ($command, $timeout, $cmdFile) = @_;
    $timeout ||= 60;

    my $rc = 0;
    my $crc = 0;
    my @output = ();

    if ( $cmdFile eq "" ) {
    my @cmd = <<"EOC";
#!/usr/bin/perl -w

    my  \$rc = 0;
    my  \$crc = 0;
    my  \@output = ();

     \$SIG{ALRM} = sub { \$rc = 1; die " Timeout in timeoutWithFork" }; 
    eval {
        alarm $timeout;
         \@output = `$command`;
         \$crc =  \$?;
        alarm 0;
    };
    
    print STDOUT "CRC:  \$crc \\n";
    print STDOUT  \@output;

    exit 0;
EOC

    $cmdFile = "/tmp/cmd-$ENV{USER}-$$-timeout.pl";
    open CMD, ">$cmdFile";
    print CMD @cmd;
    close CMD;
    chmod 0755, $cmdFile;
}

    open OUT, "$cmdFile |";
    while (<OUT>) {
	if ( /Timeout/ ) { $rc = 1; last; }
	else {
	    if ( /CRC:/ ) { my @m = split /\s+/; $crc = $m[1]; }
	    else { push @output, $_; }
	}
    }
    close OUT;

#gg    return [$rc, $crc, $output];
    return ($rc, $crc, $cmdFile, @output);
}


sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
