#!/usr/bin/perl

package BaseStep;

require Exporter;

use TestUtils; # just for mytimeout, until timeout is fine

=head1 NAME

BaseStep.pm
# $Id: BaseStep.pm,v 1.7 2003/09/19 15:21:06 grodid Exp $

=head1 SYNOPSIS

BaseStep.pm - Some utility methods to be inherited by step tests.

=head1 DESCRIPTION

BaseStep.pm - Some utility methods to be inherited by step tests:

getSelf() - Return the object internal variables in the $self array
setSelf() - Set some of the object internal variables in the $self array

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setSelf {
  my $self = shift;

  # Get as input the output hash ref from previous step, if any
  my ($href) =@_;

  if (defined $href) {
    # Store in the $self hash ref, the needed info from the input hash ref
    my @akeys = keys %{$href};
    foreach $key (@akeys) {
      $self->{$key} = $href->{$key};
    }
  }
}

sub getSelf {
  my $self = shift;
  my ($arg) = @_;
  return $self->{$arg};
}

# Utility function to issue a command
# What it does:
#   1-Writes the issued command to a tmp logfile
#   2-Issues the command with a timeout
#   3-Appends the output of this command to the main test logfile $self->{opts}->{log}
# Arguments:
#   $cmd: String with the command to be issued
#   $logflag: Flag to indicate wether to append the result to the main log (logflag=1, default) or not (logflag=0)
# Returns a hash ref with keys:
#   rc: Return code of the issued command
#   out: stdout+stderr of the issued command
sub runCommand {
  my $self = shift;
  my ($cmd,$logflag,$timeout) = @_;
  $timeout ||= 240;
  if (!defined $logflag) {$logflag = 1};

  return 1 if ( ! defined $cmd );

  print "runCommand(): Now running: \"$cmd\" \n";

  $head = "\n";
  $head .= "*************************************** \n";
  $head .= "* Log for: $cmd \n";
  $head .= "*************************************** \n";

  $href = mytimeout($cmd,$timeout);

  $head = $head . $href->{out};

  # Finally, if temporary log file name was given, append it to the main test logfile
  if ( $logflag != 0 ) {$self->logIt($head)}

  return $href;
}

# Utility method to append some string to the main log file and to the STEPLOG member variable:
sub logIt {
  my $self = shift;

  my ($str) = @_;

  open LOG, ">> $self->{opts}->{log}";
  print LOG $str;
  close LOG;

  # Append this log into an internal variable where we keep the whole step log
  $self->{STEPLOG} .= $str;

  return;
}

sub testLog {
  my $self = shift;

  my ($errCode,$errMsg) = @_;

  my %ret = (
             'OK' => 0,
             'FAIL' => 1,
             'WARNING' => 2,
             'XFAIL' => 3,
             'UNRESOLVED' => 4,
             'UNTESTED' => 5,
             'UNSUPPORTED' => 6
            );

  # Append this log into an internal variable where we keep the whole step log
  $self->logIt("\nTEST: [$errCode] $errMsg \n");

  # And set the test return code accordingly.
  $self->{STEPRC} = $ret{$errCode};

  return;
}

sub parseTURLs {
  my $self = shift;

  my ($str) = @_;

  # Use a hash ref with array refs components to store the output:
    $href->{nt} = 0;
    $href->{turls} = [];
    $href->{protocols} = [];
    $href->{hostnames} = [];
    $href->{paths} = [];

    while ( $str =~ m/(([^:\s\n]+):\/\/([^\/]+)\/+(.+))/g ) {
	++($href->{nt});
	push @{$href->{turls}}, $1;
	push @{$href->{protocols}}, $2;
	push @{$href->{hostnames}}, $3;
	push @{$href->{paths}}, "/" . $4;
    }

    return $href;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
