#!/usr/bin/perl

package BaseTest::CreateCopyandRegister;

use strict;
use vars qw(@ISA);
use EDGUtils;

@ISA=qw(BaseTest);

=head1 NAME

CreateCopyandRegister.pm 

=head1 SYNOPSIS

CreateCopyandRegister.pm - Copy a file on SE , Register one or more logical file names and remove  

=head1 DESCRIPTION 

B<CreateCopyandRegister.pm> This tests do a file , copy it on SE and do some registration before to remove all

=head1 AUTHOR

Eric Fede ( eric.fede@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "CreateCopyandRegister: setContext\n";

    # The command needed to run.
    $self->{commands} = ["globus-url-copy", 
			 "grid-proxy-info",
			 "edg-gridftp-rm"];

    # new option
    $self->addOption('SEhost=s',
                     'Name of the target SE.',
                     'lxshare0408.cern.ch');

    $self->addOption('VOname=s',
                     'Name of user VO.',
                     'wpsix');

    $self->addOption('RCConf=s',
                     'Path to the edg-replica-manager.conf.',
                     '/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf');
    
    $self->addOption('SEPathFile=s',
                     'Path on SE to The VO flatfile.',
                     '/tmp');


    $self->addOption('nchar=i',
                     'Number of characters on logical file.',
                     '12');
    $self->addOption('nrecord=i',
                     'Number of record to do.',
                     '1');

    $self->addOption('filesize=i',
                     'Size (Ko) of the file to copy on SE.',
                     '1');


}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "CreateCopyandRegister: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;
    print "CreateCopyandRegister: runTest\n";
    my @replicanamelist = createlogicalfilename($self->{opts}->{nrecord},$self->{opts}->{nchar});

    my $localurlname = createphysicalfilenameurl($self->{opts}->{filesize});
    my $physicalpathname = copyonSE($localurlname,$self->{opts}->{SEhost},$self->{opts}->{SEPathFile});
    my $registerback = register(@replicanamelist,$physicalpathname,$self->{opts}->{RCConf},$self->{opts}->{VOname});
    my $removeback = removelfn($self->{opts}->{RCConf});
#    my $removepfn = removepfn($physicalpathname);
    if (($removeback == 1) || ($registerback == 1)) {	
    return 1;}
    return 0;
}



# create a list of filename with a name with a name depending of the input parameter.
sub createlogicalfilename {
    
    my ($nboflogicalfile,$nbofcharactere) = @_;

    # find a random with a number of character depending of the input parameter
     my @listofname;
     my @validchars = ('A' .. 'Z', '0' .. '9');

     for (1 .. $nboflogicalfile){	
     	my $filename = '';
     	for (1 .. $nbofcharactere) {
        	# take a random position
		my $randompos = int(rand(($#validchars+1)));
        	$filename .= $validchars[$randompos];
        }
     push @listofname,$filename;
    # put the name on the lista
     }
    return @listofname;
}

sub createphysicalfilenameurl {
    my ($fileS) = @_;
    my $toto=`mkdir /tmp/$ENV{USER}-$$`;
    my $testfile="/tmp/$ENV{USER}-$$/testfile";
    my $commande;
    # Create a single file
    if ($fileS < 1000 ){
       $commande="dd if=/dev/random of=$testfile bs=1024 ibs=1024 count=$fileS  conv=sync";}
    elsif ($fileS > 1000000) {
       my $fileSS=$fileS/1000000;
       $commande="dd if=/dev/random of=$testfile bs=1024M ibs=1024M count=$fileSS  conv=sync";}
    else {
       my $fileSS=$fileS/1000;
       $commande="dd if=/dev/random of=$testfile bs=1024k ibs=1024k count=$fileSS  conv=sync";
    }
    print "CreateCopyandRegister: Creation of a file of $fileS ko\n";
    open(IN,"$commande |") or die $!;
    close IN;
    return $testfile;
}



# Do the copy on the SE
 
sub copyonSE {
    my ($urlname, $sehostname,$SEpath) = @_;
    # create on random name for the physical name on SE
    my @validchars = ('A' .. 'Z', '0' .. '9');
    my $filename = '';
    for (1 .. 10) {
      # take a random position
      my $randompos = int(rand(($#validchars+1)));
      $filename .= $validchars[$randompos];
    }

    print "CreateCopyandRegister: Copy file on SE\n";
    my $remoteurl = "gsiftp://$sehostname/$SEpath/$filename";
    print "globus-url-copy file://";
    print  $urlname;
    print " ";
    print $remoteurl;
    print "\n";
    my $result = `globus-url-copy file://$urlname $remoteurl 2>&1`;
    #my $pathonSEhost = "$sehostname/flatfiles/SE00/wpsix/$ENV{USER}-$$/$filename";
    my $pathonSEhost = "$sehostname/$SEpath/$filename";
    return $pathonSEhost;	
}



# Register all copies in the replica manager.
sub register {

    my (@logicalname) = @_;
    my $i;
    my $nbsuccess=0;	
    my $returnstatus=0;	
    my @listreturn;
    my $tmpurl = join("","srm://",$logicalname[$#logicalname-2]);
    my $RCconffile=$logicalname[$#logicalname-1];
    my $VOname=$logicalname[$#logicalname]; 
    my $toto=`touch /tmp/$ENV{USER}-$$/GUIDLIST`;
    
    for $i (0 .. $#logicalname-2) {
        print "CreateCopyandRegister: Registration on replica\n";
        print "edg-rm --vo=$VOname -i --config=$RCconffile registerFile $tmpurl -l lfn:$logicalname[$i] >> /tmp/$ENV{USER}-$$/GUIDLIST";
     	my $commandresult = `edg-rm --vo=$VOname -i --config=$RCconffile registerFile $tmpurl -l lfn:$logicalname[$i] >> /tmp/$ENV{USER}-$$/GUIDLIST`;
        my $flag = ($!) ? 0 : 1;
	if ($flag == 0){$nbsuccess += 1;}
	if ($flag != 0 ){$returnstatus = 1;}
        print "Registered on Replica With LFN : $logicalname[$i] \n";
        print $nbsuccess;
	print "/";
	print $#logicalname-1;
        print "\n";
	}
     return $returnstatus;
}


# remove all the logical file from the replica (this part should be rewrited soon)
sub removelfn {

    my ($RCconffile) = @_;
    my $i;
    my $GUIDID;
    my $commandresult;
    my $nbsuccess=0;
    # Open GUID file
    print "CreateCopyandRegister : Remove the logical file from the replica\n";
    my $GUIDpathfile="/tmp/$ENV{USER}-$$/GUIDLIST";
    open(GUIDD,$GUIDpathfile);
    $i = <GUIDD>;
    chomp($i);
    $commandresult = `edg-rm --vo=wpsix -i --config=$RCconffile deleteFile $i -a`;
    my $flag = ($!) ? 0 : 1;
    print "Remove from Replica : ";
    print "\n";


    close (GUID);
    return $flag;
}
# remove the physical file on SE
sub removepfn {
    my ($filenameurl) = @_;
    # Open GUID file
    print "CreateCopyandRegister : Remove the file on SE\n";
    print $filenameurl;
    my $commandresult = `edg-gridftp-rm  gsiftp://$filenameurl`;
    print $commandresult;
    return $commandresult;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
