#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::CECycle;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

CECycle.pm - Check that the site specific CEs are indeed operational.
# $Id: CECycle.pm,v 1.2 2003/09/26 09:04:51 grodid Exp $

=head1 SYNOPSIS

Check that the site specific CEs are indeed operational.

=head1 DESCRIPTION 

B<CECycle.pm> Taking into account 
the CEs specified in the site.conf, the test is meant to check (thru a 
edg-job-list-match operation) that these CEs are indeed accepted by the RB.
 Then a job is sent to each CE in turn, to check they are fully
alive and operational. This will replace the simple HelloWorld test,
where the CE itself was not specified.
 The summary page displays the results of the former check within the MATCHING 
step, and the latter check within the RUNNING step.

 To be used from the MainScript.


=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

#    print "CECycle: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '111');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::CECycle [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "CECycle";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List

    return 0;

}


# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    print "CECycle: setupEnvironment\n";

    return 0;
}


# Build any necessary setup for the storm.
sub buildCEList {
    my $self = shift;

#old    print "CECycle: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

## lcgce01.cern.ch:2119/jobmanager-lsf-grid

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
hotdog46.fnal.gov:2119/jobmanager-pbs-lcgqueue
lcg001.lnl.infn.it:2119/jobmanager-pbs-lcg
lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
lcgce01.gridpp.rl.ac.uk:2119/jobmanager-pbs-workq
lxshare0247.cern.ch:2119/jobmanager-pbs-workq
wn-02-29-a.cnaf.infn.it:2119/jobmanager-pbs-workq
EOCE

# @{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
# lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
# EOCE

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0302.cern.ch:2119/jobmanager-pbs-short
EOCE

    if ( $ENV{EDG_WL_UI_CONFIG_PATH} ) {
	if ( $ENV{EDG_WL_UI_CONFIG_PATH} =~ "gm03" ) {
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
bottom.phy.bris.ac.uk:2119/jobmanager-pbs-gridq
heplnx11.pp.rl.ac.uk:2119/jobmanager-pbs-S
hepbf4.ph.qmul.ac.uk:2119/jobmanager-pbs-S
gw24.hep.ph.ic.ac.uk:2119/jobmanager-pbs-bbrgrid
grid001.ct.infn.it:2119/jobmanager-pbs-short
grid0.fe.infn.it:2119/jobmanager-pbs-wn73
grid0.fe.infn.it:2119/jobmanager-pbs-short
gppce05.gridpp.rl.ac.uk:2119/jobmanager-pbs-S
gppce04.gridpp.rl.ac.uk:2119/jobmanager-pbs-edg
ccgridli08.in2p3.fr:2119/jobmanager-bqs-A
ccgridli03.in2p3.fr:2119/jobmanager-bqs-A
bbr-gate01.slac.stanford.edu:2119/jobmanager-lsf-medium
EOCE
	}
    }

## For CertTB only
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0235.cern.ch:2119/jobmanager-pbs-short
lxshare0290.cern.ch:2119/jobmanager-pbs-short
EOCE

     print STDERR " Ready ? $self->{jdlFile} $self->{shFile} \n";
#     system("echo Ready ? $self->{jdlFile} $self->{shFile} > Ready.txt");

    @{$self->{CEs}} = ();
#    $self->{CEs} = [];
    my $read;
    my $cmdlm = dgjob('list-match');
    my %CEq;
    open MCH, "$cmdlm $self->{jdlFile} |";
    while (<MCH>) {
	if ( /\*CEId\*/ ) { $read = 1; next; }
	if ( /jobmanager-/ && $read ) {
	    $_ =~ s/\s*(.*?)\s*/$1/g;
	    my ($CE, $queue) = split /\//, $_;
#	    push @{$self->{CEs}}, ($_ =~ m/(\S.*\S)/g);
#	    push @{$self->{CEs}}, $CE;
#	    chomp($queue);
	    $CEq{$CE} = $queue;
	    $read++;
	}
    }
    close MCH;
    foreach my $ce ( keys %CEq ) {
	push @{$self->{CEs}}, "$ce/$CEq{$ce}";
    }
#     open TXT, ">> Ready.txt";
     print STDERR " CEs: @{$self->{CEs}} \n";
#     close TXT;
    exit 0 if (0);

    return 0;
}


sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "CECycle: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $bid = `echo 'Arguments = "S$ENV{GGSERIE} I${index} rien"; ' >> $jdlTemp`;

    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
#    my $ce = ${$self->{CEs}}[$rnd];
    my $ce = ${$self->{CEs}}[($index%$nCE)];
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }


    return 0;
}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "CECycle: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    ## List of requested CEs
    @{$self->{CEr}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    ## Possible only when jdlFile and shFile get created
    $self->buildCEList();

    ## Filtering out ...
    my @celist = @{$self->{CEs}};
    $self->{CEs} = [];
    ## The matching list
    $self->{CEm} = [];
    foreach my $ces ( @celist ) {
##	next if ( $ces =~ 'lcgce' );  ##  TEST
	my @m = split /:/, $ces;
	print " CE: $m[0] \n";
	push @{$self->{CEm}}, $ces;
	foreach my $cer ( @{$self->{CEr}} ) {
	    if ( $cer =~ $m[0] ) { push @{$self->{CEs}}, $ces; last; }
	}
    }
    ## Selected CE list
    foreach my $ces ( @{$self->{CEs}} ) {
	print " Selected CE: $ces \n";
    }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}

sub buildXMLFile {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
#p    $self->printLocation( (caller(0))[3] );

    $self->buildXMLFile1Col();
    return 0;

}

sub createJDL {
    my ($self, $file, $script) = @_;

    print "CECycle: createJDL\n";
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
Requirements = other.GlueCEPolicyMaxCPUTime<600000;
Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

    print "CECycle: createScript\n";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
set -xv
echo "Hello from $Bin "
echo "Arguments: " \$1 \$2 \$3
date
##sleep 2700
##date
pwd
echo "-- end of $self->{type}.sh -- "
##notyet /usr/afsws/bin/tokens
##printenv
host \$HOSTNAME
host \$HOST
uname -a
ls -l
END
    ;
    close SCRIPT;
    if ( $self->{opts}->{batchSleep} ) {
	my $res = `echo 'sleep $self->{opts}->{batchSleep}' >> $file `;
    }

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

#    print "CECycle: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    ## Checking the matching ...
    my @ind = sort { $a <=> $b } keys %{$self->{reqs}};
    my $lind = $#ind;
    $lind++;
    my $lfile = "$self->{tempdir}/matching.out";
    my $stat1 = "[OK]";
    if ( scalar(@{$self->{CEr}}) != scalar(@{$self->{CEs}}) ) { $stat1 = "[FAIL]"; }
    $self->{reqs}->{$lind}->{stat1} = "$stat1";
    $self->{reqs}->{$lind}->{mawn} = "MATCHING";
    if ( $stat1 eq "[FAIL]" ) { $self->{reqs}->{$lind}->{olog} = "$lfile"; }
    else { $self->{reqs}->{$lind}->{olout} = "$lfile"; }
    open LF, ">$lfile";
    print LF "\n REQUESTED: \n\n";
    foreach my $ces ( @{$self->{CEr}} ) {
	$self->{CEu}->{$ces} = "[OK]";
	print LF " Requested CE: $ces \n";
    }
    print LF "\n SELECTED: \n\n";
    foreach my $ces ( @{$self->{CEs}} ) {
	print LF " selected CE: $ces \n";
    }
    close LF;

    ## Checking the running ...
    $lind++;
    $lfile = "$self->{tempdir}/running.out";
    $stat1 = "[OK]";
    open LF, ">$lfile";
    print LF "\n SELECTED: \n\n";
    foreach my $ces ( @{$self->{CEs}} ) {
	print LF " selected CE: $ces \n";
    }
    print LF "\n FOUND OK: \n\n";
    foreach my $index ( @ind ) {
	my $mace = $self->{reqs}->{$index}->{mace};
##	$self->{reqs}->{$index}->{stat1} = "[FAIL]" if ( $mace =~ 'lcgce' && $index == 3 );  ##  TEST
#	print LF " CE found running: $mace \n" if ( $self->{reqs}->{$index}->{stat1} eq "[OK]" );
	$stat1 = "[FAIL]" if ( $self->{reqs}->{$index}->{stat1} eq "[FAIL]" );
	if ( $self->{CEu}->{$mace} eq "[OK]" ) { $self->{CEu}->{$mace} = "$self->{reqs}->{$index}->{stat1}"; }
    }
    foreach my $ceu ( sort keys %{$self->{CEu}} ) {
	print LF " CE found $self->{CEu}->{$ceu}: $ceu \n"
    }
    close LF;
    $self->{reqs}->{$lind}->{stat1} = "$stat1";
    $self->{reqs}->{$lind}->{mawn} = "RUNNING";
    if ( $stat1 eq "[FAIL]" ) { $self->{reqs}->{$lind}->{olog} = "$lfile"; }
    else { $self->{reqs}->{$lind}->{olout} = "$lfile"; }

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }
    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
