#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::CheckStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

CheckStorm.pm - Check that globusrun authentication works on the hosts
# $Id: CheckStorm.pm,v 1.6 2003/09/11 15:12:19 grodid Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<CheckStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    print "CheckStorm: setContext\n";

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '135');
    $self->addOption ("cksumSize=i", 
		      "Size of the file for checksum test (Mb)", 
		      2000);


    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::CheckStorm [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "checkStorm";
	$self->{brfinfo} = "SandBox Transfers + CheckSum";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List
    $self->{bigFile} = "";

    return 0;

}



# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    print "CheckStorm: setupEnvironment\n";

    return 0;
}


# Build any necessary setup for the storm.
sub buildCEList {
    my $self = shift;

#old    print "CheckStorm: setupEnvironment\n";

## lcgce01.cern.ch:2119/jobmanager-lsf-grid

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
hotdog46.fnal.gov:2119/jobmanager-pbs-lcgqueue
lcg001.lnl.infn.it:2119/jobmanager-pbs-lcg
lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
lcgce01.gridpp.rl.ac.uk:2119/jobmanager-pbs-workq
lxshare0247.cern.ch:2119/jobmanager-pbs-workq
wn-02-29-a.cnaf.infn.it:2119/jobmanager-pbs-workq
EOCE

# @{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
# lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
# EOCE

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0302.cern.ch:2119/jobmanager-pbs-short
EOCE

    if ( $ENV{EDG_WL_UI_CONFIG_PATH} ) {
	if ( $ENV{EDG_WL_UI_CONFIG_PATH} =~ "gm03" ) {
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
bottom.phy.bris.ac.uk:2119/jobmanager-pbs-gridq
heplnx11.pp.rl.ac.uk:2119/jobmanager-pbs-S
hepbf4.ph.qmul.ac.uk:2119/jobmanager-pbs-S
gw24.hep.ph.ic.ac.uk:2119/jobmanager-pbs-bbrgrid
grid001.ct.infn.it:2119/jobmanager-pbs-short
grid0.fe.infn.it:2119/jobmanager-pbs-wn73
grid0.fe.infn.it:2119/jobmanager-pbs-short
gppce05.gridpp.rl.ac.uk:2119/jobmanager-pbs-S
gppce04.gridpp.rl.ac.uk:2119/jobmanager-pbs-edg
ccgridli08.in2p3.fr:2119/jobmanager-bqs-A
ccgridli03.in2p3.fr:2119/jobmanager-bqs-A
bbr-gate01.slac.stanford.edu:2119/jobmanager-lsf-medium
EOCE
	}
    }

## For CertTB only
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0235.cern.ch:2119/jobmanager-pbs-short
lxshare0290.cern.ch:2119/jobmanager-pbs-short
EOCE

     print STDERR " Ready ? $self->{jdlFile} $self->{shFile} \n";
#     system("echo Ready ? $self->{jdlFile} $self->{shFile} > Ready.txt");

    @{$self->{CEs}} = ();
#    $self->{CEs} = [];
    my $read;
    my $cmdlm = dgjob('list-match');
    my %CEq;
    open MCH, "$cmdlm $self->{jdlFile} |";
    while (<MCH>) {
	if ( /\*CEId\*/ ) { $read = 1; next; }
	if ( /jobmanager-/ && $read ) {
	    $_ =~ s/\s*(.*?)\s*/$1/g;
	    my ($CE, $queue) = split /\//, $_;
#	    push @{$self->{CEs}}, ($_ =~ m/(\S.*\S)/g);
#	    push @{$self->{CEs}}, $CE;
#	    chomp($queue);
	    $CEq{$CE} = $queue;
	    $read++;
	}
    }
    close MCH;
    foreach my $ce ( keys %CEq ) {
	push @{$self->{CEs}}, "$ce/$CEq{$ce}";
    }
#     open TXT, ">> Ready.txt";
     print STDERR " CEs: @{$self->{CEs}} \n";
#     close TXT;
    exit 0 if (0);

#     $self->{bigFile} = $self->{tempdir}."/BigOrig.crap";
#     my $cbf = $self->createBig($self->{bigFile});

    return 0;
}


sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

    print "CheckStorm: addArguments\n";

## Create the checksum file ...
    $self->{reqs}->{$index}->{bigo} = $self->{tempdir}."/BigOrig-$index.crap";
    my $checksum1 = $self->createBig($self->{reqs}->{$index}->{bigo});
    $checksum1 =~ s/ /-/;
    $checksum1 = "=$checksum1=";

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} $checksum1"; ' >> $jdlTemp`;
    $bid = `echo '#BigFileName = "mumbojombo"; ' >> $jdlTemp`;
    $bid = `echo 'InputSandbox = {"$self->{shFile}","$self->{reqs}->{$index}->{bigo}"}; ' >> $jdlTemp`;

    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce = ${$self->{CEs}}[$rnd];
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }


    return 0;
}


# Create a BigFile for checksuming ...
sub createBig {
    my $self = shift;
    my $bigOrig = shift;

    my $size = $self->{opts}->{cksumSize};  ## in Mb
##    $size = 2;  ## temp
##    my $time = $size*70000;
    open BGF, "> $bigOrig";
    my $bgf = "";
    for ( my $t=0; $t<$size; $t++ ) {
	my $r = rand($t);
	print BGF "$r";
	$bgf .= "$r";
    }
    print BGF "\n";
    $bgf .= "\n";
    my $cumul = 70000;
    for ( my $t=0; $t<$cumul; $t++ ) {
	print BGF "$bgf\n";
    }
    close BGF;
    #system("ls -l $bigOrig; cksum $bigOrig");
    my $checksum = `cksum $bigOrig | cut -d" " -f1-2`;
    chomp($checksum);

    return $checksum;

}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

    print "CheckStorm: setupTest\n";

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    ## Possible only when jdlFile and shFile get created
    $self->buildCEList();

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

    print "CheckStorm: createJDL\n";
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
##Arguments = "a 2 c";
##nomore InputSandbox = {"/u/eb/grodid/public/globus/grodid/edg-test/script.sh"};
##chk InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
##chk OutputSandbox = {"result.out","result.err"};
OutputSandbox = {"result.out","result.err","mumbojombo"};
##OutputSE = "bbr-gate02.slac.stanford.edu";
##bad Rank = other.MaxCpuTime;
##nomore Requirements  = Member ( other.RunTimeEnvironment , "BABAR-Test-slac-01");
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

    print "CheckStorm: createScript\n";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl -w

my ($a, $b, $c) = @ARGV;
print STDERR "checkSum: runTest ARGS: $a $b $c\n";
print STDOUT "checkSum: runTest ARGS: $a $b $c\n";
print STDOUT "=checksum1= $c\n";
my $bigOrig = `ls -1 *BigOrig*`;
my $checksum2 = `cksum $bigOrig | cut -d" " -f1-2`;
    $checksum2 =~ s/ /-/;
    $checksum2 = "=$checksum2=";
print STDOUT "=checksum2= $c\n";
open BGO, "$bigOrig";
open MBJ, "> mumbojombo";
if (1) {
# Reverse
my @lines = reverse <BGO>;
foreach my $line (@lines) {
    print MBJ "$line";
}
} else {
# Direct
while (<BGO>) {
    print MBJ;
}
}
close BGO;
close MBJ;

my $service = `date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; uname -a; ls -l`;
print STDOUT $service;

exit 0;

__DATA__

#!/bin/sh
set -xv
echo "Hello from $Bin "
echo "Arguments: " \$1 \$2 \$3
date
##sleep 2700
##date
pwd
echo "-- end of $self->{type}.sh -- "
##notyet /usr/afsws/bin/tokens
##printenv
host \$HOSTNAME
host \$HOST
uname -a
ls -l
END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

    print "CheckStorm: evaluateTest\n";

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }

    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
