#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::DataStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

DataStorm.pm - Check that globusrun authentication works on the hosts
# $Id: DataStorm.pm,v 1.2 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<DataStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

#    print "DataStorm: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '135');
#     $self->addOption ("cksumSize=i", 
# 		      "Size of the file for checksum test (Mb)", 
# 		      2000);


    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::DataStorm [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "DataStorm";
	$self->{brfinfo} = "ERM Data Transfers from WNs";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List
#    $self->{bigFile} = "";

    return 0;

}



# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

#    print "DataStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

    chomp(my $res = `grep defaultSE /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf`);
    my @m = split /=/, $res;
    $self->{targetSE} = "$m[1]";

    return 0;
}


# Build any necessary setup for the storm.
sub buildCEList {
    my $self = shift;

#old    print "DataStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

## lcgce01.cern.ch:2119/jobmanager-lsf-grid

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
hotdog46.fnal.gov:2119/jobmanager-pbs-lcgqueue
lcg001.lnl.infn.it:2119/jobmanager-pbs-lcg
lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
lcgce01.gridpp.rl.ac.uk:2119/jobmanager-pbs-workq
lxshare0247.cern.ch:2119/jobmanager-pbs-workq
wn-02-29-a.cnaf.infn.it:2119/jobmanager-pbs-workq
EOCE

# @{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
# lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
# EOCE

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0302.cern.ch:2119/jobmanager-pbs-short
EOCE

    if ( $ENV{EDG_WL_UI_CONFIG_PATH} ) {
	if ( $ENV{EDG_WL_UI_CONFIG_PATH} =~ "gm03" ) {
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
bottom.phy.bris.ac.uk:2119/jobmanager-pbs-gridq
heplnx11.pp.rl.ac.uk:2119/jobmanager-pbs-S
hepbf4.ph.qmul.ac.uk:2119/jobmanager-pbs-S
gw24.hep.ph.ic.ac.uk:2119/jobmanager-pbs-bbrgrid
grid001.ct.infn.it:2119/jobmanager-pbs-short
grid0.fe.infn.it:2119/jobmanager-pbs-wn73
grid0.fe.infn.it:2119/jobmanager-pbs-short
gppce05.gridpp.rl.ac.uk:2119/jobmanager-pbs-S
gppce04.gridpp.rl.ac.uk:2119/jobmanager-pbs-edg
ccgridli08.in2p3.fr:2119/jobmanager-bqs-A
ccgridli03.in2p3.fr:2119/jobmanager-bqs-A
bbr-gate01.slac.stanford.edu:2119/jobmanager-lsf-medium
EOCE
	}
    }

## For CertTB only
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0235.cern.ch:2119/jobmanager-pbs-short
lxshare0290.cern.ch:2119/jobmanager-pbs-short
EOCE

     print STDERR " Ready ? $self->{jdlFile} $self->{shFile} \n";
#     system("echo Ready ? $self->{jdlFile} $self->{shFile} > Ready.txt");

    @{$self->{CEs}} = ();
#    $self->{CEs} = [];
    my $read;
    my $cmdlm = dgjob('list-match');
    my %CEq;
    open MCH, "$cmdlm $self->{jdlFile} |";
    while (<MCH>) {
	if ( /\*CEId\*/ ) { $read = 1; next; }
	if ( /jobmanager-/ && $read ) {
	    $_ =~ s/\s*(.*?)\s*/$1/g;
	    my ($CE, $queue) = split /\//, $_;
#	    push @{$self->{CEs}}, ($_ =~ m/(\S.*\S)/g);
#	    push @{$self->{CEs}}, $CE;
#	    chomp($queue);
	    $CEq{$CE} = $queue;
	    $read++;
	}
    }
    close MCH;
    foreach my $ce ( keys %CEq ) {
	push @{$self->{CEs}}, "$ce/$CEq{$ce}";
    }
#     open TXT, ">> Ready.txt";
     print STDERR " CEs: @{$self->{CEs}} \n";
#     close TXT;
    exit 0 if (0);

#     $self->{bigFile} = $self->{tempdir}."/BigOrig.crap";
#     my $cbf = $self->createBig($self->{bigFile});

    return 0;
}


sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "DataStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

## Create the checksum file ...
#     $self->{reqs}->{$index}->{bigo} = $self->{tempdir}."/BigOrig-$index.crap";
#     my $checksum1 = $self->createBig($self->{reqs}->{$index}->{bigo});
#     $checksum1 =~ s/ /-/;
#     $checksum1 = "=$checksum1=";

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} $self->{targetSE}"; ' >> $jdlTemp`;
#     $bid = `echo '#BigFileName = "mumbojombo"; ' >> $jdlTemp`;
#     $bid = `echo 'InputSandbox = {"$self->{shFile}","$self->{reqs}->{$index}->{bigo}"}; ' >> $jdlTemp`;

    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce = ${$self->{CEs}}[$rnd];
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }

    return 0;
}


# Create a BigFile for checksuming ...
sub createBig {
    my $self = shift;
    my $bigOrig = shift;

    my $size = $self->{opts}->{cksumSize};  ## in Mb
##    $size = 2;  ## temp
##    my $time = $size*70000;
    open BGF, "> $bigOrig";
    my $bgf = "";
    for ( my $t=0; $t<$size; $t++ ) {
	my $r = rand($t);
	print BGF "$r";
	$bgf .= "$r";
    }
    print BGF "\n";
    $bgf .= "\n";
    my $cumul = 70000;
    for ( my $t=0; $t<$cumul; $t++ ) {
	print BGF "$bgf\n";
    }
    close BGF;
    #system("ls -l $bigOrig; cksum $bigOrig");
    my $checksum = `cksum $bigOrig | cut -d" " -f1-2`;
    chomp($checksum);

    return $checksum;

}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "DataStorm: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    ## Possible only when jdlFile and shFile get created
    $self->buildCEList();

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

#    print "DataStorm: createJDL\n";
    $self->printLocation( (caller(0))[3] );
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err", "BrokerInfo"};
##OutputSE = "bbr-gate02.slac.stanford.edu";
##??DataAccessProtocol = "file";
VirtualOrganisation = "$self->{opts}->{vo}";
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

#    print "DataStorm: createScript\n";
    $self->printLocation( (caller(0))[3] );

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl -w

my ($a, $b, $c) = @ARGV;
print STDERR "DataStorm: runTest ARGS: $a $b $c\n";
print STDOUT "DataStorm: runTest ARGS: $a JobIndex: $b returnSE: $c\n";
my $returnSE = $c;

my $res = `cp .BrokerInfo BrokerInfo 2>&1`;
my (@CSE) = `edg-brokerinfo getCloseSEs 2>&1`;
    chomp(my $theCE = `edg-brokerinfo getCE 2>&1 | cut -d":" -f 1`);
    chomp(my $theWN = `hostname --long 2>&1`);
chomp(my $closeSE = $CSE[0]);
chomp(my $myVO = `edg-brokerinfo getVirtualOrganization 2>&1`);
    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$myVO";
    print STDOUT " DataStorm: edgRMopts: $edgRMopts WN: $theWN CE: $theCE closeSE: $closeSE returnSE: $c \n";

my ($cres, $file) = createNewData($edgRMopts, $closeSE, $returnSE, $a, $b, "");
print " JobLFN: lfn:$file \n" unless $cres;

my $service = `echo; date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; uname -a; echo; ls -l`;
print STDOUT $service;

exit $cres;

sub createNewData {
    my ($edgRMopts, $SE1, $SE2, $serie, $index, $subDir) = @_;

#    $self->printLocation( (caller(0))[3] );

    ## Create another temporary data file ...
    my $file = "$ENV{USER}-$$-dstorm-$serie-$index.data";
    ## Some Pre-Cleaning
    my $rc = removeGhostFile($edgRMopts, $file);
    return $rc if $rc;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

    open TMP, ">$tmpdir/$file";
    print TMP "Good Morning to all Everybody ! This is a DataStorm. Job# $index \n";
    close TMP;

    print " JobLFN0: lfn:$file \n";

#     my @kSEpairs = keys %{$self->{SEpairs}};
#     my ($SE1, $SE2) = split /\s+/, $kSEpairs[int(rand(scalar(@kSEpairs)))];

#     my $SP1 = "$self->{SPs}->{$SE1}/$subDir";
#     my $SP2 = "$self->{SPs}->{$SE2}/$subDir";

    $rc = createReplicas ($edgRMopts, $SE1, $SE2, $subDir, $file);
    return $rc if $rc;

    return (0, "$file");
}

## Dynamic creation of Replicas job by job ...
sub createReplicas {

#     my ($self, $SE1, $SE2, $subDir, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $SE1, $SE2, $subDir, $file) = @_;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

#     my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s $self->{hostname}//tmp/${file} -d $SE1/$SP1/$file";
#     print " CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     my $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
#     if ( $stat != 0 ) {
# 	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
# 	return 1;
#     }

    my $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1";
    print "\n CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
#	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    my %PFNs = ();
    $PFNs{"$SE1/$file"} = "created";

#    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    my @resRcLR = `$command 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
	if ( $i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE1/${file}";
	if ( $csrm ne $gsrm ) {
	    print STDERR " File: ${file} on: $SE1 : A ghost file still existed: $gsrm \n";
	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE1: $SE1 $file value: $l \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

if ( "$SE1" ne "$SE2" ) {
#    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    $command = "edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2";
    my $resRcRF = `$command 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    }
    $PFNs{"$SE2/$file"} = "created";
}

#       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    @resRcLR = `$command 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
	if ( $i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( $csrm ne $gsrm ) {
	    next;
# 	    print STDERR " File: ${file} on: $SE2 : A ghost file still existed: $gsrm \n";
# 	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE2 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE2: $SE2 $file value: $l \n";
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    return 0;

}

sub removeGhostFile {

#     my ($self, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $file) = @_;

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager $edgRMopts deleteFile --all $resRcLR";
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

__DATA__

#additional storage here

END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

#    print "DataStorm: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{vo}";

    ## Implement proper parsing of output files, and subsequent evaluation here.
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#     }

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#p	print " JOUT\t$index :\n".`cat $self->{reqs}->{$index}->{olout}`." \n" if $self->{reqs}->{$index}->{olout};
	next unless $self->{reqs}->{$index}->{olout};
	chomp(my $res = `grep JobLFN0 $self->{reqs}->{$index}->{olout}`);
	my @m = split /\s+/, $res;
	print " JLFN:\t$m[2] \n";
	my $file = $m[2];
	my $match = 0;
	my $command = "edg-replica-manager $edgRMopts listReplicas ${file}";
	my @resRcLR = `$command 2>&1`;
	print "\n resRcLR0: $file -$?- =$#resRcLR= \n@resRcLR";
	foreach my $l ( @resRcLR ) {
	    chomp($l);
	    if ( $l =~ "$self->{targetSE}" ) {
		print " Matching: [OK] $l \n";
		$match++;
	    }
	}
##test	if ( $index == 1 ) { $match = 10; }  ##  TEMP TEST
	if ( $match != 1 ) { print " Matching: [FAILED] \n"; }
#old	$self->{reqs}->{$index}->{match} = $match;
	if ( $self->{reqs}->{$index}->{stat1} eq '[OK]' ) {
	    if ( $match != 1 ) {
		$self->{reqs}->{$index}->{stat1} = '[FAIL]';
		$self->{reqs}->{$index}->{stat2} = 'MATCHING';
		$self->{reqs}->{$index}->{olog} = $self->{reqs}->{$index}->{olout};
	    }
	}
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
    }

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);

    return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
