#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::RepStorm;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

RepStorm.pm - Check that globusrun authentication works on the hosts
# $Id: RepStorm.pm,v 1.6 2003/09/11 15:12:16 grodid Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<RepStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    print "RepStorm: setContext\n";

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '5');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '456');

    $self->addOption('subDir=s',
		     'Remote subdirectory name where to create the replicas.',
                     'testGGrepStorm');
#     $self->addOption('nolocation',
#                      'Flag to AVOID forcing initialization of the location stuff'
# 		     );
#     $self->addOption('VO=s',
# 		     'VO to use in the repStorm.',
#                      'wpsix');
#                     'tstg');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			 "edg-vo-env",
			 "edg-gridftp-rm",
                         "edg-gridftp-ls",
			 "edg-gridftp-mkdir",
		    "edg-replica-manager"
# 			 "gdmp_host_subscribe",
# 			 "gdmp_job_status",
# 			 "globus-replica-catalog",
# 			 "ldapsearch",
# 			 "edg-replica-manager-replicateFile",
# 			 "edg-replica-manager-copyAndRegisterFile",
# 			 "edg-replica-manager-deleteFile",
# 			 "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::RepStorm [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "repStorm";
	$self->{brfinfo} = "RM Storm + Execution";
	$self->{subDir} = "";
	$self->{location} = "";
#	$self->{VO} = "";
        $self->{dataFL} = [];      ## List
	$self->{SPs} = {};         ## Hash
	$self->{SEs} = [];         ## List
	$self->{SEpairs} = {};     ## Hash

    return 0;

}


# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

#    print "RepStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

#     my $rc = getVOtstgEnvV();
#     return $rc if $rc;

#     $rc = getRCgdmpEnvV();
#     return $rc if $rc;

    return 0;
}


# Build any necessary setup for the storm.
sub stormSetup {
    my $self = shift;

#    print "RepStorm: stormSetup\n";
    $self->printLocation( (caller(0))[3] );

    my $rc = $self->repStormSetup($self->{subDir});
    return $rc if $rc;

    return 0;
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "RepStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $subDir = $self->{subDir};
    my ($rc, $dataF) = $self->createNewData($index, $subDir);
    return $rc if $rc;

    push @{$self->{dataFL}}, $dataF;
    my $bid = `echo '    InputData = "lfn:${dataF}";' >> $jdlTemp`;
    $bid =    `echo '    Arguments = "lfn:${dataF}";' >> $jdlTemp`;

    $self->{reqs}->{$index}->{mace} = "free";

    return $rc;
}

sub evaluateTest {
    my $self = shift;

#    print "JobStorm: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);

#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
#     }

    return 0;

}

# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "RepStorm: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    $self->{subDir} = $self->{opts}->{subDir};
    if ( $self->{opts}->{nolocation} == 1 ) { $self->{location} = 'nothing'; }
    else { $self->{location} = 'needed'; }
#    print " Location: $self->{location} $self->{subDir} \n";
#old    $self->{VO} = $self->{opts}->{VO};
    $self->{opts}->{VO} = $self->{opts}->{vo};
    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    ## Default options to be used for all ERM related commands ...
    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{VO}";
    print " edgRMopts: $edgRMopts \n";
    $self->{edgRMopts} = $edgRMopts;

    return $rc;
}


# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

#    print "RepStorm: cleanup\n";
    $self->printLocation( (caller(0))[3] );
    return 0 if (1);  ##  TEMP

    ## Cleanup temp dataFiles in both local and SEs
    my $resClean = $self->dataFClean($self->{subDir}, @{$self->{dataFL}});

    my $rc = $self->SUPER::cleanup();

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

#    print "RepStorm: createJDL\n";
    $self->printLocation( (caller(0))[3] );
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
    Executable    = "$sname";
    StdOutput     = "std.out";
    StdError      = "std.err";
    InputSandbox  = {"$script"};
    OutputSandbox = {"std.out","std.err"};
    StdInput = "/dev/null";
    VirtualOrganisation = "$self->{opts}->{VO}";
    DataAccessProtocol = "file";
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

#    print "RepStorm: createScript\n";
    $self->printLocation( (caller(0))[3] );

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/bin/bash
set -x
echo Arg: $1
GSF=`edg-brokerinfo getSelectedFile $1 file`
echo GSF: $GSF
TFN=`echo $GSF | cut -d " " -f 2`
echo Printing local file $TFN:
cat $TFN
echo "+-+-+-+-+-+-+-+-"
/bin/echo "Hello From Skript"
/bin/echo "Goodbye. [OK]"
GSE=`edg-brokerinfo getSEs`
echo SEs: $GSE
CSE=`edg-brokerinfo getCloseSEs`
echo getCloseSEs: $CSE
CSE=`echo $CSE | cut -d " " -f 1`
SMP=`edg-brokerinfo getSEMountPoint $CSE`
echo getSEMountPoint: $SMP

F1="/"`edg-brokerinfo getLFN2SFN $1 | grep $CSE | cut -d"/" -f4-`
echo "+-+-+-+-+-+-+-+-"
echo LFN 1 $1 $F1
contents=`cat $F1`
echo LFC 1 $contents
echo "+-+-+-+-+-+-+-+-"

/usr/bin/host $HOSTNAME
/usr/bin/host $HOST
/bin/uname -a
END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

## Mutual subscription check for SPs
sub mutualSPs {
    my ($self, $subDir) = @_;
    my $rc = 0;
    my %SEpairs = ();

    ## Mutual subscription test ...
    foreach my $se1 ( @{$self->{SEs}} ) {
	foreach my $se2 ( @{$self->{SEs}} ) {
	    next if ( $se1 eq $se2 );
	    $SEpairs{"$se1 $se2"} = 1;
	}
    }
    foreach my $k ( keys %SEpairs ) {
	if ( $SEpairs{$k} == 0 ) {
	    delete $SEpairs{$k};
	}
    }
    if ( scalar(keys(%SEpairs)) == 0 ) {
	print STDERR " NO SEpair available ! \n";
	return 1;
    } else {
	print "\nPossible SEpairs: \n", map { "$_ => $SEpairs{$_}\n" } keys %SEpairs;
    }


    return $rc if $rc;

    %{$self->{SEpairs}} = %SEpairs;
    return 0;

}

sub repStormSetup {
    my ($self, $subDir) = @_;
    print " ARGS: @{$self->{args}} \n";

# #     my $rc = getVOtstgEnvV();
# #     return $rc if $rc;

#     my $rc = $self->getSPs();
#     return $rc if $rc;

    print "  Working this SE list: @{$self->{SEs}} \n";
#     foreach my $se ( @{$self->{SEs}} ) {
#         print "     FS: $se => $self->{SPs}->{$se} \n";
#     }
    
    my $rc = $self->mutualSPs($subDir);
    return $rc if $rc;

    return 0;

}

sub createNewData {
    my ($self, $index, $subDir) = @_;

    $self->printLocation( (caller(0))[3] );

    ## Create another temporary data file ...
    my $file = "$ENV{USER}-$$-repStorm-$index.data";
    ## Some Pre-Cleaning
    my $rc = $self->removeGhostFile($file);
    return $rc if $rc;

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};

    open TMP, ">$tmpdir/$file";
    print TMP "Good Morning to all Everybody ! This is a repStorm. Job# $index \n";
    close TMP;

    my @kSEpairs = keys %{$self->{SEpairs}};
    my ($SE1, $SE2) = split /\s+/, $kSEpairs[int(rand(scalar(@kSEpairs)))];
#     my $SP1 = "$self->{SPs}->{$SE1}/$subDir";
#     my $SP2 = "$self->{SPs}->{$SE2}/$subDir";

    $rc = $self->createReplicas ($SE1, $SE2, $subDir, $file);
    return $rc if $rc;

    return (0, "$file");
}

## Dynamic creation of Replicas job by job ...
sub createReplicas {

    my ($self, $SE1, $SE2, $subDir, $file) = @_;
    my $edgRMopts = $self->{edgRMopts};

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};

#     my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s $self->{hostname}//tmp/${file} -d $SE1/$SP1/$file";
#     print " CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     my $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
#     if ( $stat != 0 ) {
# 	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
# 	return 1;
#     }

    my $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1";
    print "\n CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
#	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    my %PFNs = ();
    $PFNs{"$SE1/$file"} = "created";

#    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    my @resRcLR = `$command 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
	if ( $i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE1/${file}";
	if ( $csrm ne $gsrm ) {
	    print STDERR " File: ${file} on: $SE1 : A ghost file still existed: $gsrm \n";
	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE1: $SE1 $file value: $l \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

#    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    $command = "edg-replica-manager $edgRMopts replicateFile lfn:${file} -d $SE2";
    my $resRcRF = `$command 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    }
    $PFNs{"$SE2/$file"} = "created";

#       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    @resRcLR = `$command 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
	if ( $i[0] ne "srm" ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( $csrm ne $gsrm ) {
	    next;
# 	    print STDERR " File: ${file} on: $SE2 : A ghost file still existed: $gsrm \n";
# 	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE2 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE2: $SE2 $file value: $l \n";
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    return 0;

}


## To extract the SE Mount Points and all that ...
sub getSPs {
    my $self = shift;

    my %SPs= ();
    my %SPst= ();

    ## Check that "tstg" VO exists on all SEs ...
    foreach my $se ( @{$self->{SEs}} ) {
        my $command = "ldapsearch -x -h $se -p 2135 -b \"mds-vo-name=local,o=grid\" \"objectclass=storageelement\" sevo | grep -i sevo:";
        my $arrayref = timeout($command, 60);
        my @output = @$arrayref;
#        my $output = `$command`;
#        my @lines = split("\n","$output\n");
    # Catch errors from the execution of the command.
        if ($output[0]==1) {
            $self->setErrorMessage('could not fork ldapsearch');
            return 1;
        } elsif ($output[0]==2) {
            $self->setErrorMessage('ldapsearch timed out on SE $se');
            return 1;
        }
	my @lines = split("\n","$output[1]\n");
        my @voex = grep { /tstg/ } @lines;
        print " VOEX: $se @voex \n";
        if ( $#voex != 0 ) {
            return 1;
        }
        my @p = split /\s+/, $voex[0];
        @p = split /:/, $p[1];
        if ( $p[0] ne "tstg" ) {
            return 1;
        }
        $SPs{$se} = $p[1];

        ## Another check about these mount points ...
        my $gdmpSt = `gdmp_job_status -S $se -c gdmp.conf | grep GDMP_STORAGE_DIR`;
#p      print " GDMPST: $se $gdmpSt";
        @p = split /=/, $gdmpSt;
        $SPst{$se} = $p[1];
        chomp($SPst{$se});
        if ( $SPst{$se} ne $SPs{$se} ) {
            print STDERR " Mount point mismatch on SE $se: $SPst{$se} $SPs{$se} \n";
            return 1;
        }

    }
    print "SPs: \n", map { "$_ => $SPs{$_}\n" } keys %SPs;

    %{$self->{SPs}} = %SPs;

    return 0;

}


sub dataFClean {
    my ($self, $subDir, @dataFL) = @_;
    my $edgRMopts = $self->{edgRMopts};

    my %PFNs = ();

    foreach my $file ( @dataFL ) {

    ## Delete local file ...
    unlink "/tmp/$file";
##    next if 1;  ##temp

    ## Delete SE files and also RM entry
    my @resRcDF = grep { /deleting/ } `edg-replica-manager-deleteFile -e -a -l ${subDir}/${file} 2>&1`;
    print "\n resRcDF3: $file -$?- =$#resRcDF= \n@resRcDF";
    foreach my $l ( @resRcDF ) {
	chomp $l;
        my @i = split /deleting /, $l;
        $PFNs{$i[1]} = "deleting";
    }

    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    print "\n resRcLR3: $file -$?- =$#resRcLR= \n";
    if ( $#resRcLR > -1 ) {
	foreach my $l ( @resRcLR ) {
	    chomp $l;
	    my @i = split /: /, $l;
	    $PFNs{$i[1]} = "still-existing";
	}
    }

}

    print "\nPFNs: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";

## Also delete the directory on each SE
    foreach my $se ( @{$self->{SEs}} ) {
	my $sp = $self->{SPs}->{$se};
	my $resDel = `edg-gridftp-rm gsiftp://${se}${sp}/$subDir`;
    }

    return 0;

}

sub removeGhostFile {

    my ($self, $file) = @_;
    my $edgRMopts = $self->{edgRMopts};

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager $edgRMopts deleteFile --all $resRcLR";
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
