#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::UserStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

UserStorm.pm - Check that globusrun authentication works on the hosts
# $Id: UserStorm.pm,v 1.3 2003/10/03 14:34:31 grodid Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<UserStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '811');
    $self->addOption('customJDL=s',
                     'File name of the custom JDL file.');
    $self->addOption('customScript=s',
                     'File name of the custom SCRIPT file.');
    $self->addOption('circular',
                     'To scan circularly the CEList built with useCEList');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::UserStorm [options]

END
    ;

    $self->{DescText} = <<'END'

CHANGE THIS TEXT ...
This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "UserStorm";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List

    return 0;

}


# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    return 0;
}


# Build any necessary setup for the storm.
sub buildCEList {
    my $self = shift;

#old    print "UserStorm: setupEnvironment\n";

## lcgce01.cern.ch:2119/jobmanager-lsf-grid

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
hotdog46.fnal.gov:2119/jobmanager-pbs-lcgqueue
lcg001.lnl.infn.it:2119/jobmanager-pbs-lcg
lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
lcgce01.gridpp.rl.ac.uk:2119/jobmanager-pbs-workq
lxshare0247.cern.ch:2119/jobmanager-pbs-workq
wn-02-29-a.cnaf.infn.it:2119/jobmanager-pbs-workq
EOCE

# @{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
# lcg00103.grid.sinica.edu.tw:2119/jobmanager-pbs-workq
# EOCE

@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0302.cern.ch:2119/jobmanager-pbs-short
EOCE

    if ( $ENV{EDG_WL_UI_CONFIG_PATH} ) {
	if ( $ENV{EDG_WL_UI_CONFIG_PATH} =~ "gm03" ) {
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
bottom.phy.bris.ac.uk:2119/jobmanager-pbs-gridq
heplnx11.pp.rl.ac.uk:2119/jobmanager-pbs-S
hepbf4.ph.qmul.ac.uk:2119/jobmanager-pbs-S
gw24.hep.ph.ic.ac.uk:2119/jobmanager-pbs-bbrgrid
grid001.ct.infn.it:2119/jobmanager-pbs-short
grid0.fe.infn.it:2119/jobmanager-pbs-wn73
grid0.fe.infn.it:2119/jobmanager-pbs-short
gppce05.gridpp.rl.ac.uk:2119/jobmanager-pbs-S
gppce04.gridpp.rl.ac.uk:2119/jobmanager-pbs-edg
ccgridli08.in2p3.fr:2119/jobmanager-bqs-A
ccgridli03.in2p3.fr:2119/jobmanager-bqs-A
bbr-gate01.slac.stanford.edu:2119/jobmanager-lsf-medium
EOCE
	}
    }

## For CertTB only
@{$self->{CEs}} = <<"EOCE" =~ m/(\S.*\S)/g;
lxshare0235.cern.ch:2119/jobmanager-pbs-short
lxshare0290.cern.ch:2119/jobmanager-pbs-short
EOCE

     print STDERR " Ready ? $self->{jdlFile} $self->{shFile} \n";
#     system("echo Ready ? $self->{jdlFile} $self->{shFile} > Ready.txt");

    @{$self->{CEs}} = ();
#    $self->{CEs} = [];
    my $read;
    my $cmdlm = dgjob('list-match');
    my %CEq;
    open MCH, "$cmdlm $self->{jdlFile} |";
    while (<MCH>) {
	if ( /\*CEId\*/ ) { $read = 1; next; }
	if ( /jobmanager-/ && $read ) {
	    $_ =~ s/\s*(.*?)\s*/$1/g;
	    my ($CE, $queue) = split /\//, $_;
#	    push @{$self->{CEs}}, ($_ =~ m/(\S.*\S)/g);
#	    push @{$self->{CEs}}, $CE;
#	    chomp($queue);
	    $CEq{$CE} = $queue;
	    $read++;
	}
    }
    close MCH;
    foreach my $ce ( keys %CEq ) {
	push @{$self->{CEs}}, "$ce/$CEq{$ce}";
    }
#     open TXT, ">> Ready.txt";
     print STDERR " CEs: @{$self->{CEs}} \n";
#     close TXT;
    exit 0 if (0);

    return 0;
}


sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

    $self->printLocation( (caller(0))[3] );

    my $bid;
#    $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} rien"; ' >> $jdlTemp`;

    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce;
    if ( $self->{opts}->{circular} ) {
	$ce = ${$self->{CEs}}[($index%$nCE)];
    } else {
	$ce = ${$self->{CEs}}[$rnd];
    }
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }


    return 0;
}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    @{$self->{SEs}} = @{$self->{args}};

    if ( ! ( $self->{opts}->{customJDL} || $self->{opts}->{customScript} ) ) {
	$self->setErrorMessage('you must specify one of customJDL or customScript options');
	return 1;
    }
    if ( $self->{opts}->{customJDL} && $self->{opts}->{customScript} ) {
	$self->setErrorMessage('options customJDL and customScript are mutually exclusive');
	return 1;
    }
    if ( $self->{opts}->{customJDL} && ! -s $self->{opts}->{customJDL} ) {
	$self->setErrorMessage('custom JDL file does not exist');
	return 1;
    }
    if ( $self->{opts}->{customScript} && ! -s $self->{opts}->{customScript} ) {
	$self->setErrorMessage('custom Script file does not exist');
	return 1;
    }

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();

    ## Possible only after jdlFile and shFile get created
    $self->buildCEList();  ##  ?? do we need this here as well ?

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}

sub createJDL {
    my ($self, $jdlFile, $scriptFile) = @_;

    $self->printLocation( (caller(0))[3] );
    print " jdlFile scriptFile: $jdlFile $scriptFile \n";

    my ($mname, $mpath, $msuffix) = fileparse($scriptFile, []);

    my $uScript;
    my $uJDL;
    my $sname;
##    my $script;
    my $sist;
    my $isb;
    my $storeUJDL;

    ## Case for customScript provided

    if ( $self->{opts}->{customScript} ) {
	$uScript = $self->{opts}->{customScript};
	my ($fname, $fpath, $suffix) = fileparse($uScript, []);
	$sname = $fname;
##	$script = $uScript;

    # Write out the job JDL description.
    open JDL, ">$jdlFile";
    print JDL <<"END"
Executable = "$mname";
##Arguments = "a 2 c";
InputSandbox = {"$uScript", "$scriptFile"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
END
    ;
    close JDL;

    ## Case for customJDL provided

    } elsif ( $self->{opts}->{customJDL} ) {
	$uJDL = $self->{opts}->{customJDL};
	open JDL, "<$uJDL";
	while (<JDL>) {
	    $storeUJDL .= $_;
	    # Executable = "sname";
	    if ( /Executable/ ) {
#p		print " EE: $_ \n";
		$_ =~ /"(.*)"/ ;
		$sname = $1;
	    }
	}
	close JDL;
	open JDL, "<$uJDL";
	while (<JDL>) {
	    # InputSandbox = {"script","scrap"};
	    if ( /InputSandbox/ ) {
		chomp($isb = $_);
		$_ =~ /\{(.*)\}/ ;
		$sist = $1;
#p    	        print " II: $sist\n";
	        my @list = split /,/, $sist;
	    foreach my $it ( @list ) {
		$it =~ /"(.*)"/ ;
		$sist = $1;
		my ($fname, $fpath, $suffix) = fileparse($sist, []);
		if ( $fname eq $sname ) { $uScript = $sist; }
	    }
	    }
	}
	close JDL;
	if ( $uScript && ! -s $uScript ) { 
	    $self->setErrorMessage('custom Script taken out from custom JDL does not exist');
	    return;
	}

	my @isbs = split /}/, $isb;
        $isb = $isbs[0] . ', "' . "$scriptFile" . '"};';
        chomp($storeUJDL);

    # Write out the job JDL description, 
    # while adding the NEW Executable and InputSandbox clauses
    open JDL, ">$jdlFile";
    print JDL <<"END"
$storeUJDL
Executable = "$mname";
$isb
END
    ;
    close JDL;

    }

    print STDOUT "\n Custom: E=$sname S=$uScript \n\n";

    if ( ! ( $sname && $uScript ) ) { 
	$self->setErrorMessage('settings extracted from customScript/customJDL are wrong');
	return;
    }
    $self->{sname} = "$sname";
    $self->{uScript} = "$uScript";
    $self->{uJDL} = "$uJDL" if $uJDL;

    # Write out the job description.
    open SCRIPT, ">$scriptFile";
    print SCRIPT <<"END"
#!/usr/bin/perl -w

print STDERR "userStorm: ARGS: \@ARGV\\n";
print STDOUT "userStorm: ARGS: \@ARGV\\n";
system("ls -l; host \$ENV{HOSTNAME}; echo; echo '======> Begin execution user script'; echo");
chmod 0755, "./$self->{sname}";

system("./$self->{sname} \@ARGV");
my \$stat = \$?;

if ( ! \$stat ) {
    my \$res = `echo; echo '======> End execution user script'; echo; date; pwd; host \$ENV{HOSTNAME}; uname -a; ls -l`;
    print STDOUT \$res;
}

exit 0;
END
    ;

    return "$jdlFile";

}

sub createScript {
    my ($self, $file) = @_;

    $self->printLocation( (caller(0))[3] );

    ## Quite dummy now, in this UserStorm subclass only
    ## since everything is built in the createJDL method 

    return "$file";

}

# This will run the evaluateTest method on all jobs.
sub evaluateTest {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }
    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
