#!/usr/bin/perl

package BaseTest::Globusrun;

use strict;
use vars qw(@ISA);

@ISA=qw(BaseTest);

=head1 NAME

Globusrun.pm - Check that globusrun authentication works on the hosts
# $Id: Globusrun.pm,v 1.4 2003/10/01 12:22:20 grodid Exp $

=head1 SYNOPSIS

Globusrun.pm - Check that globusrun authentication works on the hosts.

=head1 DESCRIPTION 

B<Globusrun.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

#    print "ReverseDNS: setContext\n";
    $self->printLocation( (caller(0))[3] );

    # Commands necessary to run this test.
    $self->{commands} = ["globusrun", 
			 "grid-proxy-info"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::Globusrun [options] <hostname> [hostname ...]

END
    ;

    $self->{DescText} = <<'END'

Check that a reverse DNS lookup works for all of the supplied 
host names.   The host names must be fully qualified.

The reverse DNS lookup is done as a basic part of checking the 
validity of host certificates.  If the lookup does not work, then
secured requests to that machine will fail (e.g. job submission,
GridFTP transfers, etc.).

END
    ;

}

sub runTest {
    my $self = shift;

#    print "ReverseDNS: runTest\n";
    $self->printLocation( (caller(0))[3] );

    # Keep a list of all of the host names which fail.
    my @failed;

    # Loop over each argument doing a reverse DNS lookup for each.
    foreach my $host (@{$self->{args}}) {
        print "Checking... $host\n";

	my $command = "globusrun -a -r $host";
	my @output = timeout($command, 60);
        # Forward direction to get IP number.
        my ($hname, $aliases, $addrtype, $length, @binaddrs) = 
	    gethostbyname($host);
        if ($?) {
	    print "Failed in forward direction. $?\n";
            push @failed, $host;
        } else {

            # Reverse to get the name.  It must return exactly the 
            # same hostname!
            $hname = gethostbyaddr($binaddrs[0], $addrtype);
	    if ("$hname" ne "$host") {
		print "Failed in reverse direction. ($hname != $host)\n";
		push @failed, $host;
	    }
	}
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("reverse DNS lookup failed for $names");
        return 1;
    }

    return 0;
}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

#    print "EDGLifecycle: checkPrerequisites\n";
    $self->printLocation( (caller(0))[3] );

       $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;

