#!/usr/bin/perl

package BaseTest::RM::RMAltCopyRegister;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);
use TestUtils;

=head1 NAME

RMAltCopyRegister.pm  - Copy and register a file, and then do an integrity check

=head1 SYNOPSIS

Copy and register a file, and then do an integrity check

=head1 DESCRIPTION 

B<RMAltCopyRegister.pm> This script is inteded to be called from a RM test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    $self->{commands} = ["edg-replica-manager-copyFile","edg-replica-manager-registerEntry","edg-gridftp-exists","edg-replica-manager-cd","edg-replica-manager-pwd","edg-replica-manager-ls","edg-replica-manager-cat"];
  } else {
    $self->{commands} = ["edg-replica-manager"];
    #+DBG
    print "Needed commands not yet coded for RMAltCopyRegister edg2! EXIT";
    exit 1;
    #-DBG
  }

  $self->{SHORTINFO} = "Copy and register";
  $self->{LONGINFO} = "print some long info here...";
}

# The test step script goes here
sub runTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  my $date = sprintf("%02d%02d%02d%02d%02d%02d", $year % 100,$mon + 1,$mday,$hour,$min,$sec);
  # Store the test FileName as a global variable
  $self->{TESTFILENAME} = "test_file_RM." . $date;

  # Create a local random file with the required size
  $self->runCommand("dd if=/dev/urandom of=/tmp/$self->{TESTFILENAME} bs=1024 count=$self->{opts}->{filesize}",0);

  # Logical File Name
  my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};

  # Issue RM command
  my $href = $self->runCommand("edg-replica-manager-copyFile -s `hostname`/tmp/$self->{TESTFILENAME} -d $aHosts[0]$self->{PATHS}->{$aHosts[0]}/$self->{TESTFILENAME} -c $self->{opts}->{rcconf}");

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-replica-manager-copyFile command failed with ret val $href->{rc}");
    return;
  }
  # Issue RM command
  my $href = $self->runCommand("edg-replica-manager-registerEntry -s $aHosts[0]$self->{PATHS}->{$aHosts[0]}/$self->{TESTFILENAME} -l $lfn -c $self->{opts}->{rcconf}");

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-replica-manager-registerEntry command failed with ret val $href->{rc}");
    return;
  }

  return;
}

# Override the BaseTest::evaluateTest method
sub evaluateTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  # Logical File Name
  my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};

  my $href = $self->runCommand("edg-gridftp-exists gsiftp://$aHosts[0]$self->{PATHS}->{$aHosts[0]}/$self->{TESTFILENAME}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-cd / -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-cd $self->{opts}->{rdir} -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-pwd");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-ls -1 -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $rPath="";
  if ($href->{out} =~ /($self->{TESTFILENAME})/){$rPath=$1;}
  if ($rPath ne $self->{TESTFILENAME}) {
    $self->testLog("FAIL","Replica not found on RC");
    return;
  }
# WARNING 

  my $href = mytimeout ("edg-replica-manager-cat $self->{TESTFILENAME} -c $self->{opts}->{rcconf} &> /tmp/$self->{TESTFILENAME}.replica");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }

  my $href = $self->runCommand("diff /tmp/$self->{TESTFILENAME} /tmp/$self->{TESTFILENAME}.replica");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","File integrity check failed");
    return;
  }

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
