#!/usr/bin/perl

package BaseTest::RM::RMAltDelete;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);

=head1 NAME

RMAltDelete.pm - Deletes all replicas of a file, and checks it

=head1 SYNOPSIS

Deletes all replicas of a file, and checks it

=head1 DESCRIPTION 

B<RMAltDelete.pm> This script is inteded to be called from a ReplicaManager test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    $self->{commands} = ["edg-replica-manager-unregisterEntry","edg-gridftp-rm","edg-gridftp-exists","edg-replica-manager-cd","edg-replica-manager-pwd","edg-replica-manager-ls","edg-replica-manager-cat"];
  } else {
    $self->{commands} = ["edg-replica-manager"];
    #+DBG
    print "Needed commands not yet coded for RMAlt Delete edg2! EXIT";
    exit 1;
    #-DBG
  }

  $self->{SHORTINFO} = "Delete";
  $self->{LONGINFO} = "print some long info here...";
}

# The test step script goes here
sub runTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};

  my $href = $self->runCommand("edg-replica-manager-unregisterEntry -a -l $lfn -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-replica-manager-unregisterEntry command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-gridftp-rm gsiftp://$aHosts[0]$self->{PATHS}->{$aHosts[0]}/$self->{TESTFILENAME}");
  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-gridftp-rm command failed on $aHosts[0]  with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-gridftp-rm gsiftp://$aHosts[1]$self->{PATHS}->{$aHosts[1]}/$self->{TESTFILENAME}");
  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-gridftp-rm command failed on $aHosts[1]  with ret val $href->{rc}");
    return;
  }
  return;
}


# Override the BaseTest::evaluateTest method
sub evaluateTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};
  $self->{STEPRC} = 0;

  # Logical File Name
  my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};

  my $href = $self->runCommand("edg-replica-manager-cd / -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-cd $self->{opts}->{rdir} -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-pwd");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $href = $self->runCommand("edg-replica-manager-ls -1 -c $self->{opts}->{rcconf}");
  if ($href->{rc} != 0){
    $self->testLog("FAIL","Command failed with ret val $href->{rc}");
    return;
  }
  my $rPath="";
  if ($href->{out} =~ /($self->{TESTFILENAME})/){$rPath=$1;}
  if ($rPath eq $self->{TESTFILENAME}) {
    $self->testLog("FAIL","File still registered on RC");
    return;
  }

  # Check that the files do not exist in the SEs anymore:
  foreach $host (@aHosts) {
  my $href = $self->runCommand("edg-gridftp-exists gsiftp://${host}$self->{PATHS}->{$host}/$self->{TESTFILENAME}");
    if ($href->{rc} == 0) {
      $self->testLog("FAIL","File still present on SE");
      return;
    }
  }

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
