#!/usr/bin/perl

package BaseTest::RM::RMCompare;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);

=head1 NAME

RMCompare.pm - Copies the replicated file to the local host and compares to the original file

=head1 SYNOPSIS

Copies the replicated file to the local host and compares to the original file

=head1 DESCRIPTION 

B<RMCompare.pm> This script is inteded to be called from a ReplicaManager test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;
  $self->SUPER::setContext();

  $self->{commands} = ["edg-replica-manager"];

  $self->{SHORTINFO} = "Compare";
  $self->{LONGINFO} = "edg-rm copyFile SURL2 FILE + diff";
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose copyFile $self->{SURL2} file:///tmp/$self->{TESTFILENAME}.2");

  if ($href->{rc} != 0){
    $self->testLog("FAIL","copyFile command failed with ret val $href->{rc}");
    return;
  }

  return;
}

sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  $href = $self->runCommand("/usr/bin/diff /tmp/$self->{TESTFILENAME} /tmp/$self->{TESTFILENAME}.2");
  if ($href->{rc} != 0) {
    $self->testLog("FAIL","Files are different! Ret val $href->{rc}");
    return;
  }

  # Cleanup at the end
  `rm -rf /tmp/$self->{TESTFILENAME} /tmp/$self->{TESTFILENAME}.2`;

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
