#!/usr/bin/perl

package BaseTest::RM::RMCopyAndRegister;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);

=head1 NAME

RMCopyAndRegister.pm  - Copy and register a file, and then do an integrity check
# $Id: RMCopyAndRegister.pm,v 1.9 2003/09/19 15:21:06 grodid Exp $

=head1 SYNOPSIS

Copy and register a file, and then do an integrity check

=head1 DESCRIPTION 

B<RMCopyAndRegister.pm> This script is inteded to be called from a RM test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

    $self->printLocation( (caller(0))[3] );

  $self->SUPER::setContext();

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    $self->{commands} = ["edg-replica-manager-copyAndRegisterFile"];
  } else {
    $self->{commands} = ["edg-replica-manager"];
  }

  $self->{SHORTINFO} = "Copy and register";
  $self->{LONGINFO} = "edg-rm copyAndRegisterFile FILE -d HOST1 -l LFN";
  $self->{SAVEVARS} = ["TESTFILENAME","GUID","SURL1"];
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  my @aHosts = @{$self->{args}};

  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  my $date = sprintf("%02d%02d%02d%02d%02d%02d", $year % 100,$mon,$mday,$hour,$min,$sec);
  # Store the test FileName as a global variable
  $self->{TESTFILENAME} = "test_file_RM." . $date;

  # Create a local random file with the required size
  $self->runCommand("dd if=/dev/urandom of=/tmp/$self->{TESTFILENAME} bs=1024 count=$self->{opts}->{filesize}",0);

  # Issue the RM command (care of version)
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager-copyAndRegisterFile -s `hostname`/tmp/$self->{TESTFILENAME} -d $aHosts[0] -l $lfn -c $self->{opts}->{rcconf}");
  } else {
    # It is not clear to me wether it makes sense to have subdirs in the LFN...
    my $lfn = "lfn:" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose copyAndRegisterFile file:/tmp/$self->{TESTFILENAME} -d $aHosts[0] -l $lfn");

    # Parse the copyAndRegisterFile and store the returned GUID
    if ($href->{out} =~ /(guid:([0-9a-z]|-)+)/) {
      $self->{GUID} = $1;
    } else {
      $self->testLog("FAIL","GUID not found in the copyAndRegisterFile output!");
      return;
    }
  }

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","copyAndRegisterFile command failed with ret val $href->{rc}");
    return;
  }

  return;
}

# Override the BaseTest::evaluateTest method
sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  my @aHosts = @{$self->{args}};

  # Issue the RM command (care about version)
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager-listReplicas -l $lfn -c $self->{opts}->{rcconf}");
  } else {
    my $lfn = "lfn:" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose listReplicas $lfn");
  }

  if ($href->{rc} != 0){
    $self->testLog("FAIL","listReplicas command failed with ret val $href->{rc}");
    return;
  }

  # Parse the output of the previous command and look for a replica in hostA
  my $ifound = 0;
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    if ($href->{out} =~ /location[^:]*:\s*($aHosts[0])(.+)/) {
      $rPath = $2;
      $ifound = 1;
    }
  } else {
    my $href = $self->parseTURLs($href->{out});
    foreach my $prot (@{$href->{protocols}}) {
	my $surl = shift @{$href->{turls}};
	if ($prot eq 'srm') {
	    $self->{SURL1} = $surl;
	    $ifound = 1;
	    last;
	}
    }
  }

  if ($ifound == 0) {
    $self->testLog("FAIL","Replica not found at host $aHosts[0]");
    return;
  }

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
