#!/usr/bin/perl

package BaseTest::RM::RMCycleSetup;

require BaseStep;

@ISA = qw(BaseStep BaseTest::RM);

=head1 NAME

RMCycleSetup.pm - Does all necessary startup for a RM replication cycle.

=head1 SYNOPSIS

Does all necessary startup for a RM replication cycle.

=head1 DESCRIPTION 

B<RMCycleSetup.pm> This script is inteded to be called from a ReplicaManager test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["grid-info-search",
		       "edg-gridftp-exists",
		       "edg-gridftp-mkdir"];

  $self->{SHORTINFO} = "Setup";
  $self->{LONGINFO} = "print some long info here...";
  $self->{SAVEVARS} = ["PATHS"];
}

# The test step script goes here
sub runTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  # Create an empty hash ref where to store the remote SE /flatfiles paths
  # NOTE: The hash keys will be the respective SE hostnames
  $self->{PATHS} = {};

  foreach $host (@aHosts) {
    # Query the SE GRIS to get the /flatfiles path
    my ($rc,$path) = $self->grisQuery($host);
    if ($rc != 0) {
      $self->{STEPRC} = 1; return;
    }

    # Append the chosen remote dir name to the SE path and store in the hash ref $self->{PATHS} for later use
    $path = $path . "/" . $self->{opts}->{rdir};
    $self->{PATHS}->{$host} = $path;

    # Make the destination subdir in the SE, if it does not exist yet
    my $href = $self->runCommand("edg-gridftp-exists gsiftp://$host$path");

    if ($href->{rc} != 0) {
      my $href = $self->runCommand("edg-gridftp-mkdir gsiftp://$host$path");
      if ($href->{rc} != 0) {
	$self->testLog("FAIL",'error in edg-gridftp-mkdir');
	return;
      }
    }
  }

  return;
}

# Utility function to Query a SE GRIS
# Arguments:
#  $host: Host SE to which issue the LDAP MDS query
# Returns an array of 2 scalars: ret value and /flatfiles path in the remote SE
sub grisQuery {
  my $self = shift;

  my ($host) = @_;

  my $rc = 0;

  my $href = $self->runCommand("grid-info-search -x -h $host -p 2135 -b mds-vo-name=local,o=grid objectclass=StorageElement SEvo");

  # Parse the SE GRIS query and look for the /flatfiles path
  if ($href->{out} =~ /[^:]+:\s*$self->{opts}->{vo}:(.+)/) {$pathHost = $1;}

  if (!defined $pathHost) {
    $self->testLog("FAIL","ERROR: $host does not support $self->{opts}->{vo} vo");
    $rc = 1;
  }

  return ($rc,$pathHost);
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
