#!/usr/bin/perl

package BaseTest::RM::RMDelete;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);

=head1 NAME

RMDelete.pm - Deletes all replicas of a file, and checks it

=head1 SYNOPSIS

Deletes all replicas of a file, and checks it

=head1 DESCRIPTION 

B<RMDelete.pm> This script is inteded to be called from a ReplicaManager test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    $self->{commands} = ["edg-replica-manager-deleteFile"];
  } else {
    $self->{commands} = ["edg-replica-manager"];
  }

  $self->{SHORTINFO} = "Delete";
  $self->{LONGINFO} = "edg-rm deleteFile GUID --all";
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  # Issue the RM command (care of version)
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager-deleteFile -a -l $lfn -c $self->{opts}->{rcconf}");
  } else {
    $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose deleteFile $self->{GUID} --all");
  }

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","deleteFile command failed with ret val $href->{rc}");
    return;
  }

  return;
}


# Override the BaseTest::evaluateTest method
sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  my @aHosts = @{$self->{args}};

  # Issue the RM command (care about version)
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager-listReplicas -l $lfn -c $self->{opts}->{rcconf}");
  } else {
    my $lfn = "lfn:" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose listReplicas $lfn");
  }

  if ($href->{rc} != 0){
##    $self->testLog("FAIL","listReplicas command failed with ret val $href->{rc}");
## GM - 19/09/03 - Seems that if the replicas do not exist, the command returns "java.io.FileNotFoundException:" and RC=255. This is ok for us.
    $self->testLog("OK","listReplicas command failed with ret val $href->{rc}. This should mean the replicas have been deleted ok...");
    return;
  }

  # Parse the output of the previous command and check that there are no replicas
  my $patt;
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    $patt = "location";
  } else {
    $patt = "srm:\/\/";
  }

  if ($href->{out} =~ /$patt/) {
    $self->testLog("FAIL","List of replicas found NOT empty!");
    return;
  }

  # Check that the files do not exist in the SEs anymore:
  foreach $host (@aHosts) {
    # GM: For the moment, this is for edg1. For edg2, try to use the getTurl command...
    if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
      $href = $self->runCommand("edg-gridftp-exists gsiftp://${host}$self->{PATHS}->{$host}/$self->{TESTFILENAME}");
      if ($href->{rc} == 0) {
	$self->testLog("FAIL","The file is still there!");
	return;
      }
    }
  }

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
