#!/usr/bin/perl

package BaseTest::RM::RMMssDeleteFile;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);

=head1 NAME

RMDelete.pm - Deletes a file from SE, to force MSS staging

=head1 SYNOPSIS

Deletes a file from SE, to force MSS staging

=head1 DESCRIPTION 

B<RMMssDeleteFile.pm> This script is inteded to be called from a ReplicaManager test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["edg-gridftp-rm"];

  $self->{SHORTINFO} = "MssDeleteFile";
  $self->{LONGINFO} = "print some long info here...";
}

# The test step script goes here
sub runTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  my $lfn = "gsiftp://" . @aHosts[0] . "/" . $self->{PATHS}->{@aHosts[0]} . "/" . $self->{TESTFILENAME};

  my $href= $self->runCommand("edg-gridftp-rm $lfn");

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-gridftp-rm command failed with ret val $href->{rc}");
    return;
  }
  return;
}

sub evaluateTest {
  my $self = shift;

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  # Logical File Name

  my $host=$aHosts[0];
  $self->runCommand("edg-gridftp-exists gsiftp://${host}$self->{PATHS}->{$host}/$self->{TESTFILENAME}");
  if ($href->{rc} == 0) {
    $self->testLog("FAIL","It seems that the file is still there!");
    return;
  }
  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
