#!/usr/bin/perl

package BaseTest::RM::RMReplicate;

require BaseStep;

@ISA=qw(BaseStep BaseTest::RM);

=head1 NAME

RMReplicate.pm - Replicates a file from an SE to another SE
# $Id: RMReplicate.pm,v 1.10 2003/09/19 15:21:06 grodid Exp $

=head1 SYNOPSIS

Replicates a file from an SE to another SE

=head1 DESCRIPTION 

B<RMReplicate.pm> This script is inteded to be called from a ReplicaManager test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    $self->{commands} = ["edg-replica-manager-replicateFile"];
  } else {
    $self->{commands} = ["edg-replica-manager"];
  }

  $self->{SHORTINFO} = "Replicate";
  $self->{LONGINFO} = "edg-rm replicateFile SURL1 -d HOST2";
  $self->{SAVEVARS} = ["SURL2"];
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  my @aHosts = @{$self->{args}};

  $self->{STEPRC} = 0;

  # Issue the RM command (care of version)
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager-replicateFile -s $aHosts[0] -d $aHosts[1] -l $lfn -c $self->{opts}->{rcconf}");
  } else {
    my $lfn = "lfn:" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose replicateFile $self->{SURL1} -d $aHosts[1]");
  }

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","replicateFile command failed with ret val $href->{rc}");
    return;
  }

  # For EDG 2.0, parse the stdout of replicateFile and get the SURL:
  if ( $ENV{EDG_FLAVOR} eq 'edg2' ) {
      my $ifound = 0;
      my $href = $self->parseTURLs($href->{out});
      foreach my $prot (@{$href->{protocols}}) {
	  my $host = shift @{$href->{hostnames}};
	  my $surl = shift @{$href->{turls}};
	  if ($prot eq 'srm' && $host eq $aHosts[1]) {
	      $self->{SURL2} = $surl;
	      $ifound = 1;
	      last;
	  }
      }      
      if ($ifound == 0) { 
	  $self->testLog("FAIL","Could not parse SURL in replicateFile cmd stdout");
      }
  }

  return;
}

sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  my @aHosts = @{$self->{args}};

  # Issue the RM command (care of version)
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    my $lfn = $self->{opts}->{rdir} . "/" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager-listReplicas -l $lfn -c $self->{opts}->{rcconf}");
  } else {
    my $lfn = "lfn:" . $self->{TESTFILENAME};
    $href = $self->runCommand("edg-replica-manager $self->{CONFIG} --vo=$self->{opts}->{vo} $self->{INSECURE} --verbose listReplicas $lfn");
  }

  if ($href->{rc} != 0){
    $self->testLog("FAIL","listReplicas command failed with ret val $href->{rc}");
    return;
  }

  # Parse the output of the previous command and look for a replica in hostA and hostB
  if ( ! defined $ENV{EDG_FLAVOR} || $ENV{EDG_FLAVOR} eq 'edg1' ) {
    foreach $host (@aHosts) {
      if ($href->{out} =~ /location[^:]*:\s*($host)(.+)/) {$rPath = $2;}
      if (!defined $rPath) {
	$self->testLog("FAIL","Replica not found at host $host");
	return;
      } else {
	$self->testLog("OK","Replica found at host $host");
      }
    }

    # Now compare checksums of the local and the hostB remote files
    $href = $self->checkSumTest($aHosts[1],$rPath,"/tmp/$self->{TESTFILENAME}");
    if ($href->{rc} != 0) {
      $self->testLog("FAIL",$href->{err});
      return;
    }

  } else {
    foreach $surl ($self->{SURL1},$self->{SURL2}) {
      if ($href->{out} =~ /$surl/) {
	$self->testLog("OK","Replica found for SURL $surl");
      } else {
	$self->testLog("FAIL","Replica not found for SURL $surl");
	return;
      }
    }
  }

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
