#!/usr/bin/perl

package BaseTest::SEws;

use EDGUtils;

require BaseStep; ## NB I only add BaseStep here for the getSelf method invocation from TestCycle! Add getSelf to BaseTest and remove this!
@ISA=qw(BaseTest BaseStep); ## IDEM

=head1 NAME

RM.pm

=head1 SYNOPSIS

SEws.pm - BaseTest derived class containing common stuff for any
        edg-se-webservice CLI tests

setContext() - Common option flags for SEws tests.
checkArguments() - Checks common arguments for SEws tests.
checkPrerequisites() - Deals with common option flags, etc.

=head1 DESCRIPTION

SEws.pm - BaseTest derived class containing common stuff for any
        edg-se-webservice CLI tests

setContext() - Common option flags for SEws tests.
checkArguments() - Checks common arguments for SEws tests.
checkPrerequisites() - Deals with common option flags, etc.

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;
  $self->SUPER::setContext();

  print "SEws::setContext()\n";

  # Default value for savevars array ref: empty.
  $self->{SAVEVARS} = [];

  # Number of args to run SEws tests: 1 Storage Element
  $self->{NARGS} = 1;

  # Add the filesize option
  $self->addOption('filesize=i',
		   'Size in kB of the file used for replication. (Default: 256)',
		   256);

  # Add the logfile option
  $self->addOption('log=s','Logfile','SEws.log');

  # Add the --insecure option
  $self->addOption('insecure','Connect in an insecure manner, i.e. not https','');

  # Add the --timeout option
  $self->addOption('timeout=i','Timeout for the issued commands. (Default: 240 sec)',240);

  # Add the --vo option
  $self->addOption('vo=s','Virtual Organisation','');
}

# Check that mandatory arguments have been given
sub checkArguments {
    my $self = shift;

    my $error = $self->SUPER::checkArguments();
    if ($error) {
        $self->setErrorMessage('standard argument checking failed');
        return 1;
    }

    # ?GM After parsing args for a step, I see that $self->{args} still contains
    #     all the option flags, while ARGV has been truncated by GetOptions and
    #     has only arguments. I don't quite understand why this happens.
    #     Anyhow, I want to ensure that from here on $self->{args} has only arguments.
    @{$self->{args}} = @ARGV;

    # Ensure that one SE hostname have been specified.
    if ($#{$self->{args}} + 1 < $self->{NARGS} ) {
      print "ERROR - Wrong number of arguments found, expect at least $self->{NARGS}\n";
      print "<@{$self->{args}}>\n";
      return 1;
    }

    return 0;
}

# Check that:
#  -The mandatory options have been given
#  -The Grid proxy exists
sub checkPrerequisites {
  my $self = shift;
  my $rc = $self->SUPER::checkPrerequisites();
  return $rc if ($rc);

  # Ensure that the mandatory options have been given
#  if (!defined $self->{opts}->{vo}) {
#    $self->setErrorMessage("Mandatory option -vo was not provided");
#    return 1;
#  };

  # Check that Grid proxy exists
  my $rc = checkProxyExists();
  $self->setErrorMessage('user proxy does not exist') if ($rc);

  return $rc;
}

sub runTest {
  my $self = shift;
  $self->{STEPRC} = 0;
  $self->{HOST} = @{$self->{args}}[0];

  if (defined $self->{opts}->{insecure}) { $self->{INSECURE} = "--insecure";}
  
  if (defined $self->{opts}->{vo}) {
      $self->{CMDOPTS} = "--vo $self->{opts}->{vo} -h $self->{HOST}";
  } else {
      my $ep = "http://" . $self->{HOST} . ":8080/edg-se-webservice/services/edg-se-webservice ";
      $self->{CMDOPTS} = "--endpoint " . $ep;
  }

  return 0;
}

sub evaluateTest {
  my $self = shift;
  $self->{HOST} = @{$self->{args}}[0];
  return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
