#!/usr/bin/perl

package BaseTest::SEws::SEwsCache;

require BaseStep;

@ISA=qw(BaseStep BaseTest::SEws);

=head1 NAME

SEwsCache.pm

=head1 SYNOPSIS

SEwsCache.pm - edg-se-webservice cache

=head1 DESCRIPTION 

B<SEwsCache.pm> This script is inteded to be called from a SE web service CLI test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["edg-se-webservice"];

  $self->{SHORTINFO} = "cache";
  $self->{LONGINFO} = "edg-se-webservice cache SFN -h HOST";
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

 # SFN (Storage file name) This contains (1) hostname, and (2) a virtual file path
  my $sfn = $self->{HOST} ."/" . $self->{TESTFILENAME};

  # Issue the command
  $href = $self->runCommand("edg-se-webservice $self->{INSECURE} cache $sfn $self->{CMDOPTS}",1,$self->{opts}->{timeout});

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","cache command failed with ret val $href->{rc}");
    return;
  }

  # Parse the output and get the TURL. Compare with the previous TURL...
  my $href = $self->parseTURLs($href->{out});

  if ($href->{nt} == 0) {
      $self->testLog("FAIL","Error parsing the output TURL");
      return;
  } else {
      # Compare turls
      # NB: I take here the 1st turl found... see if there will be more than one...
      my $turl = shift @{$href->{turls}};
      if ($self->{TURL} ne $turl) {
	  $self->testLOG("FAIL","Different TURL! \n TURL1<$self->{TURL}>\n TURL2<$turl>");
	  return;
      }
  }

  return;
}

sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  # Issue the command
  # NB: Could we use gridft-exists or se-webservice ls here?
  $href = $self->runCommand("edg-gridftp-ls --verbose $self->{TURL}",1,$self->{opts}->{timeout});

  if ($href->{out} =~ /$self->{TURLPATH}/) {
    $self->testLog("OK","edg-gridftp-ls lists the file on disk!");
  } else {
    $self->testLog("FAIL","edg-gridftp-ls did not list the file! Ret val $href->{rc}");
    return;
  }
  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
