#!/usr/bin/perl

package BaseTest::SEws::SEwsCompare;

require BaseStep;

@ISA=qw(BaseStep BaseTest::SEws);

=head1 NAME

SEwsCompare.pm

=head1 SYNOPSIS

SEwsCompare.pm - Compare the file in the remote SE with the original local copy

=head1 DESCRIPTION 

B<SEwsCompare.pm> This script is inteded to be called from a SE web service CLI test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["globus-url-copy"];

  $self->{SHORTINFO} = "compare";
  $self->{LONGINFO} = "globus-url-copy TURL FILE.2 + diff FILE FILE.2";
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  # Issue the command
  $href = $self->runCommand("globus-url-copy $self->{TURL} file:///tmp/$self->{TESTFILENAME}.2");

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","globus-url-copy command failed with ret val $href->{rc}");
    return;
  }

  return;
}

sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  # Issue the command
  # NB: Could we use gridft-exists or se-webservice ls here?
  $href = $self->runCommand("/usr/bin/diff /tmp/$self->{TESTFILENAME} /tmp/$self->{TESTFILENAME}.2");

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","Files are different! Ret val $href->{rc}");
    return;
  }

  # Cleanup at the end
  `rm -rf /tmp/$self->{TESTFILENAME} /tmp/$self->{TESTFILENAME}.2`;

  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
