#!/usr/bin/perl

package BaseTest::SEws::SEwsCopy;

require BaseStep;

@ISA=qw(BaseStep BaseTest::SEws);

=head1 NAME

SEwsCopy.pm

=head1 SYNOPSIS

SEwsCopy.pm - globus-url-copy a file into a SE

=head1 DESCRIPTION 

B<SEwsCopy.pm> This script is inteded to be called from a SE web service CLI test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["edg-se-webservice","globus-url-copy"];

  $self->{SHORTINFO} = "copy";
  $self->{LONGINFO} = "globus-url-copy FILE TURL";
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  # Create a local random file with the required size
  $self->runCommand("dd if=/dev/urandom of=/tmp/$self->{TESTFILENAME} bs=1024 count=$self->{opts}->{filesize}",1,$self->{opts}->{timeout});

  # Issue the SE command
  $href = $self->runCommand("globus-url-copy file:///tmp/$self->{TESTFILENAME} $self->{TURL}",1,$self->{opts}->{timeout});

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","globus-url-copy command failed with ret val $href->{rc}");
    return;
  }

  return;
}

sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  # Issue the SE command
  $href = $self->runCommand("edg-se-webservice $self->{INSECURE} ls $self->{HOST}/ $self->{CMDOPTS}",1,$self->{opts}->{timeout});

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","SEws ls command failed with ret val $href->{rc}");
    return;
  }

  if ($href->{out} !~ /$self->{TESTFILENAME}/) {
    $self->testLog("FAIL","File $self->{TESTFILENAME} not found in the SE");
    return;
  }
  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
