#!/usr/bin/perl

package BaseTest::SEws::SEwsCreate;

require BaseStep;

@ISA=qw(BaseStep BaseTest::SEws);

=head1 NAME

SEwsCreate.pm

=head1 SYNOPSIS

SEwsCreate.pm - Issue "edg-se-webservice create"

=head1 DESCRIPTION 

B<SEwsCreate.pm> This script is inteded to be called from a SE web service CLI test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["edg-se-webservice"];

  $self->{SHORTINFO} = "create";
  $self->{LONGINFO} = "edg-se-webservice create SFN -h HOST";
  $self->{SAVEVARS} = ["TURL","TURLPATH","TESTFILENAME"];

}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  my $date = sprintf("%02d%02d%02d%02d%02d%02d", $year % 100,$mon,$mday,$hour,$min,$sec);
  # Store the test FileName as a global variable
  $self->{TESTFILENAME} = "test_file_SEws." . $date;

  # SFN (Storage file name) This contains (1) hostname, and (2) a virtual file path
  my $sfn = $self->{HOST} ."/" . $self->{TESTFILENAME};

  #NB: It seems that there is no edg-se-webservice command to create a directory. Only "elemkdir". 
  #    So, don't use directories in this test.

  # Issue the SE command
  $href = $self->runCommand("edg-se-webservice $self->{INSECURE} create $sfn $self->{CMDOPTS}",1,$self->{opts}->{timeout});

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","SEws create command failed with ret val $href->{rc}");
    return;
  }

  # Parse the output and get the TURL:
  my $href = $self->parseTURLs($href->{out});

  if ($href->{nt} == 0) {
      $self->testLog("FAIL","Error parsing the output TURL");
      return;
  } else {
      # Store the turl in an member variable
      # NB: I take here the 1st turl found... see if there will be more than one...
      my $turl = shift @{$href->{turls}};
      my $path = shift @{$href->{paths}};
      $self->{TURL} = $turl;
      $self->{TURLPATH} = $path;      
  }
  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
