#!/usr/bin/perl

package BaseTest::SEws::SEwsMssDelete;

require BaseStep;

@ISA=qw(BaseStep BaseTest::SEws);

=head1 NAME

SEwsMssDelete.pm

=head1 SYNOPSIS

SEwsMssDelete.pm - Delete a file in a SE using edg-gridftp-rm

=head1 DESCRIPTION 

B<SEwsMssDelete.pm> This script is inteded to be called from a SE web service CLI test

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;

  $self->SUPER::setContext();

  $self->{commands} = ["edg-gridftp-rm","edg-gridftp-ls"];

  $self->{SHORTINFO} = "delete";
  $self->{LONGINFO} = "edg-gridftp-rm TURL";
}

# The test step script goes here
sub runTest {
  my $self = shift;
  $self->SUPER::runTest();

  # Issue the command
  $href = $self->runCommand("edg-gridftp-rm $self->{TURL}");

  if ($href->{rc} != 0) {
    $self->testLog("FAIL","edg-gridftp-rm command failed with ret val $href->{rc}");
    return;
  }

  return;
}

sub evaluateTest {
  my $self = shift;
  $self->SUPER::evaluateTest();

  # Issue the command
  # NB: Could we use gridft-exists or se-webservice ls here?
  $href = $self->runCommand("edg-gridftp-ls --verbose $self->{TURL}");

  if ($href->{out} =~ /$self->{TURLPATH}/) {
    $self->testLog("FAIL","edg-gridftp-ls listed the file! Ret val $href->{rc}");
    return;
  }
  return;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
