#!/usr/bin/perl

package BaseTest::TestCycle;

@ISA=qw(BaseTest);

use EDGUtils;
use File::Path;
use IO::Handle;

=head1 NAME

TestCycle.pm - BLA
# $Id: TestCycle.pm,v 1.5 2003/09/19 15:21:06 grodid Exp $

=head1 SYNOPSIS

BLA

=head1 DESCRIPTION

BLA

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
  my $self = shift;
  my $class = ref $self;

  $self->SUPER::setContext();

  print "TestCycle::setContext()\n";

  $self->setCycleVars();

  # Build the --desc text string
  $self->{DescText} .= "\n\nDifferent cycles might be chosen through the --cycle option: \n\n";
  foreach $key (sort keys %{$self->{STEPLIST}}) {
    $self->{DescText} .= "$key:  @{$self->{STEPLIST}->{$key}} \n";
  }

  # Instantiate the Base service object to inherit its specific option flags
  if ( !defined $self->{PACKAGE} ) {
    $self->setErrorMessage("TestCycle: Error \$self->{PACKAGE} should be defined!");
    return 1;
  }

  my $pkg = "BaseTest::" . $self->{PACKAGE};
  eval "require $pkg";
  die "ERROR: Can't load package ($pkg).\n\n$@\n\n" if $@;

  $obj = $pkg->new();

  @{$self->{optionLinkage}} = @{$obj->getSelf("optionLinkage")};
  @{$self->{optionDesc}} = @{$obj->getSelf("optionDesc")};
  @{$self->{optionDefaults}} = @{$obj->getSelf("optionDefaults")};

  # Inherit also the number of arguments of the tests:
  $self->{NARGS} = $obj->getSelf(NARGS);

  # Add the cycle choice option
  $self->addOption('cycle=s',
		   'Name of the cycle to be executed. For detalis see --desc. (Default: std)',
		   'std');

  # Add the xml output option
  $self->addOption('xml',
		   'Flag to enable/disable xml log file output (Default: disabled)',
		   '');
}

sub runTest {
  my $self = shift;
  my $class = ref $self;

  $self->SUPER::runTest();

  print "TestCycle::runTest()\n";

  my @aLogs = $self->{opts}->{log};

  # If selected, the xml log file will have the same name as the ordinary log file plus .xml appended
  if ($self->{opts}->{xml}) {
#    $self->{XMLLOG} = $self->{opts}->{log} . ".xml";
    $self->{XMLLOG} = $self->{xmlfile};  ##  GG
    push (@aLogs,$self->{XMLLOG});
  }

  foreach $logs (@aLogs) {
    # Touch the needed logfiles for the whole test.
    # Later, every single command will append its log to these main logfiles.
    system ("rm -rf $logs ; touch  $logs");
    print "Logfile $logs created\n";
  }

  # Check that the number of args arrives to the minimum set by the NARGS var:
   if ($#{$self->{args}} + 1 < $self->{NARGS}) {
      print "TestCycle: ERROR - Wrong number of arguments found, expect at least $self->{NARGS}\n";
      print "<@{$self->{args}}>\n";
      return 1;
    }

  @{$self->{ARGSLIST}} = @{$self->{args}};

  if ($self->{NARGS} == 2) {
    # 2D test
    @aColsList = @{$self->{ARGSLIST}};
    @aRowsList = @{$self->{ARGSLIST}};
  } else {
    # 1D test
    @aColsList = @{$self->{ARGSLIST}};
    @aRowsList = ("dummyOneElementRow");
  }

  # Launch the test cycle series here to build the table:
  foreach $row (@aRowsList) {
    foreach $col (@aColsList) {
      if ($row eq $col) {
	# For the diagonal elements, insert some dummy test results
	foreach $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {
	  $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc} = 5;
	  $self->{XMLOUT}->{$row}->{$col}->{$step}->{out} = "Step not run. Need 2 different SEs";
	}
      } else {
	# Run the cycle test
	$self->runStepCycle($row,$col);
      }
    }
  }

  my $global_retval=0;
  my $rowM = "";
  foreach $row (@aRowsList) {
    my $colM = "";
    foreach $col (@aColsList) {
      my $stepM = "";
      foreach $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {
	my $retv = $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc};
	my $logf = $self->{XMLOUT}->{$row}->{$col}->{$step}->{out};
	if ( $row ne $col && $retv != 0) {  ##  GG
#p	    print STDOUT " global_retval: $retv $row $col \n";
	    $global_retval=1; 
	}
	$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
      }
      $colM .= makeX($stepM, "col");
    }
    $rowM .= makeX($colM, "row");
  }

  # Build now the info part of the XML output
  my $stepI = "";
  foreach $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {
    my $brf_info = $self->{STEPINFO}->{$step}->{brf_info};
    my $det_info = $self->{STEPINFO}->{$step}->{det_info};
    $stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
  }

  my $body = $rowM;

  my $info = makeX($self->{SHORTINFO}, "title");
  $info .= makeX(`date`, "date");
  if ($self->{NARGS} == 2) {
    # Add the rows_list field only for 2D tests
    $info .= makeX("@aRowsList", "rows_list");
  }
  $info .= makeX("@aColsList", "cols_list");
  $info .= $stepI;
  $info = makeX($info, "info");

  ## The whole log is made of INFO+BODY
  open XML, "> $self->{XMLLOG}";
  print XML makeX($info." \n".$body, "log");
  close XML;

##GG
  my $global_result = ' [OK]';
  $global_result = ' [FAIL]' if $global_retval;
  $self->{globalresult} .= $global_result if $self->{globalresult};

  return $global_retval;
}

# Run here one step cycle and save the data needed for the XML output into the 
# hrefs: $self->{STEPINFO} and $self->{XMLOUT}
sub runStepCycle {
  my $self = shift;

  my ($row,$col) = @_;

  # Load the current set of arguments in $self->{args}
  if ($self->{NARGS} == 2) {
    # 2D test
    $self->{args} = [$row,$col];
  } else {
    # 1D test
    $self->{args} = [$col];
  }

  my @aSaveVars = ();
  my $hrefIn = {};
  my $prevRC = 0;

  # Loop on the step objects, instantiate them and call their process() method
  foreach $step (@{$self->{STEPLIST}->{$self->{opts}->{cycle}}}) {

    $pkg = "BaseTest::" . $self->{PACKAGE} . "::" . $step;
    eval "require $pkg";
    die "ERROR: Can't load package ($pkg).\n\n$@\n\n" if $@;

    # Instantiate step objects
    # NOTE: Since we are passing the args array by reference, here I force
    #       to have a different reference for each object.
    my $arTmp = $self->getStepArgs();
    $obj = $pkg->new($arTmp);

    # Store the brief and long info for each step
    $self->{STEPINFO}->{$step}->{brf_info} = $obj->getSelf(SHORTINFO);
    $self->{STEPINFO}->{$step}->{det_info} = $obj->getSelf(LONGINFO);

    # Only perform the step test if the previous step test was successful:
    if ($prevRC != 0) {
      $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc} = 5;
      $self->{XMLOUT}->{$row}->{$col}->{$step}->{out} = "UNTESTED. Previous step results unavailable.";
    } else {

      # Pass the needed vars to the $self href inside the object
      $obj->setSelf($hrefIn);

      # Call the process() method of the step object instance here
##    print "TestCycle: Call process() for <$obj>\n";
      $obj->process();

      # Store the steps output and return code in the XMLOUT href
      $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc} = $obj->getSelf(STEPRC);
      $self->{XMLOUT}->{$row}->{$col}->{$step}->{out} = $obj->getSelf(STEPLOG);

      $prevRC = $self->{XMLOUT}->{$row}->{$col}->{$step}->{rc};

      # Ask the object which variables it wants to pass to the next objects
      # and add them to the global @aSaveVars list
      $aref = $obj->getSelf(SAVEVARS);
      @aSaveVars = (@aSaveVars , @{$aref});

      # Store the vars we want to save for the next step in a hash ref
      foreach $var (@aSaveVars) {
	$hrefIn->{$var} = $obj->getSelf($var);
      }
    }
  }

##gg  return $hrOutRc;
  return 0;

}

# Utility function that rebuilds an args array (options + arguments) without the Cycle-specific options.
# We need this to feed into the step objects new() methods...
sub getStepArgs {
  my $self = shift;

  $self->{CYCLEOPTS} = ['xml','cycle'];

  my $stepArgs = [];

  my $olkStr = join (' ',@{$self->{optionLinkage}});

  # First build the option flags array, excluding thost options in the CYCLEOPTS array
  my @akeys = keys %{$self->{opts}};
  foreach $key (@akeys) {
    my $iok = 1;
    foreach $opt (@{$self->{CYCLEOPTS}}) {
      if ($key eq $opt) {
	$iok = 0;
	last;
      }
    }	
    if ($iok) {
      my $flag = "-" . $key;
      push (@{$stepArgs},$flag);

      # Only append the flag value for those flags with the "=" in the optionLinkage
      if ($olkStr =~ /\s$key=/) {
	push (@{$stepArgs},$self->{opts}->{$key});
      }
    }
  }

##  print "stepArgs<@{$stepArgs}>\n";
##  print "args<@{$self->{args}}>\n";
##  print "olkStr<$olkStr>\n";

  # Then append the arguments
  push (@{$stepArgs},@{$self->{args}});

  return $stepArgs;
}

sub setupTest {
    my $self = shift;

    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    # Create a temporary directory to hold job submission files and
    # job output.
    my $tmpdir;
    do {
        my $value = int(rand(32768));
        $tmpdir = "/tmp/$ENV{USER}-$$-$value";
    } while (-e $tmpdir);

    # Now create the directory.
    mkpath($tmpdir, 0, 0755);
    if ($?) {
        $self->setErrorMessage('could not create temporary directory');
        return 1;
    }
    print "Temporary directory: $tmpdir\n";
    $self->{tempdir} = $tmpdir;

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

## gg for XML creation and processing ...
    if ( $ENV{MAINSCRIPT} ) {
	$self->{type} = $self->{PACKAGE};
	my $base = "$self->{tempdir}";
	$self->{xmlfile}  = "$base/".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";
    }

    return 0;

}

sub buildXMLFile {
    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
