#!/usr/bin/perl

package TestUtils;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(mytimeout);

=head1 NAME

TestUtils.pm

=head1 SYNOPSIS

TestUtils.pm - Module including some utility functions used in other test objects

=head1 DESCRIPTION 

B<TestUtils.pm> Module including some utility functions used in other test objects:

mytimeout() -  A utility function to execute a given command with a timeout. This is not intended to be called directly but to be used through the edg-tb-test command.

=head1 AUTHOR

Gonzalo Merino and Miquel Barcelo.  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.

=head1 LICENSE

Copyright (c) 2002 by Gonzalo Merino and Miquel Barcelo (IFAE).
All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

# A utility function to execute a given command with a timeout
sub mytimeout {
  my ($command, $timeout) = @_;
  $timeout ||= 240;

  my $rc = 0;
  my $output = '';

  my $r = rand;

  eval {
    local $SIG{ALRM} = sub {die "time out\n"};
    alarm $timeout;
    $output = `$command 2>&1`;
    $rc = $? >> 8;
    alarm 0;
  };

  if ($@) {
    die unless $@ eq "time out\n";   # propagate unexpected errors
    # timed out
    $rc = -1;
    print "Command timed out\n";
  }

  return { rc=>$rc, out=>$output};

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
